/*
 * Copyright (c) 2011-2016, NVIDIA CORPORATION. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include <gst/gst.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <termios.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <math.h>
#include "gst/interfaces/xoverlay.h"

#ifdef USE_CAMERABIN2
#include "gst/pbutils/pbutils.h"
#include "gst/pbutils/encoding-profile.h"
#include "gst/pbutils/encoding-target.h"
#endif

#define NVGST_DEFAULT_CAPTURE_MODE           CAPTURE_IMAGE
#define NVGST_DEFAULT_SENSOR_ID              0
#define NVGST_DEFAULT_PREVIEW_WIDTH          640
#define NVGST_DEFAULT_PREVIEW_HEIGHT         480
#define NVGST_DEFAULT_PREVIEW_TIME           0
#define NVGST_DEFAULT_CAP_START_DELAY        5
#define NVGST_DEFAULT_CAPTURE_WIDTH          640
#define NVGST_DEFAULT_CAPTURE_HEIGHT         480
#define NVGST_DEFAULT_CAPTURE_TIME           10
#define NVGST_DEFAULT_QUIT_TIME              0
#define NVGST_DEFAULT_EXPOSURE               0
#define NVGST_DEFAULT_ISO                    0
#define NVGST_DEFAULT_CONTRAST               0
#define NVGST_DEFAULT_METERMODE              0
#define NVGST_DEFAULT_TEMPORAL_TRADEOFF      0
#define NVGST_DEFAULT_FLICKER                3
#define NVGST_DEFAULT_BRIGHTNESS             33
#define NVGST_DEFAULT_SATURATION             0
#define NVGST_DEFAULT_HUE                    0
#define NVGST_DEFAULT_EDGEENHANCEMENT        0.0
#define NVGST_DEFAULT_FILTER                 0
#define NVGST_DEFAULT_WHITEBALANCE           0
#define NVGST_DEFAULT_PREVIEW_ROTATION       0
#define NVGST_DEFAULT_CAPTURE_ROTATION       0
#define NVGST_DEFAULT_IMAGE_ENCODER          2
#define NVGST_DEFAULT_VIDEO_ENCODER          FORMAT_H264_HW
#define NVGST_DEFAULT_AUDIO_ENCODER          0
#define NVGST_DEFAULT_CHANNELS               2
#define NVGST_DEFAULT_SAMPLE_RATE            44100
#define NVGST_DEFAULT_IMAGE_QUALITY          90
#define NVGST_DEFAULT_VIDEO_QUALITY_LEVEL    2
#define NVGST_DEFAULT_BITRATE                4000000
#define NVGST_DEFAULT_IFRAMEINTERVAL         60
#define NVGST_DEFAULT_CAPTURE_DELAY          0
#define NVGST_DEFAULT_STEREO_MODE            0
#define NVGST_DEFAULT_RC_MODE                2
#define NVGST_DEFAULT_CAPTURE_COUNT          0
#define NVGST_SET_FLAGS                      FALSE
#define NVGST_DEFAULT_SCENE_MODE             0
#define NVGST_DEFAULT_FOCUS_MODE             0
#define NVGST_DEFAULT_ZOOM                   1.0
#define NVGST_DEFAULT_EVCOMP                 0.0
#define NVGST_DEFAULT_FOCUS_MODE             0
#define NVGST_DEFAULT_NR                     0
#define NVGST_DEFAULT_FLASH_MODE             1
#define NVGST_DEFAULT_FILE_TYPE              FILE_MP4
#define NVGST_DEFAULT_VOICE_CONTROL          VOICE_ON
#define NVGST_DEFAULT_COLOR_MODE                0
#define DEFAULT_LOCATION                     "cap_0"
#define NVGST_DEFAULT_PREVIEW_MODE           "xvimagesink"
#define NVGST_DEFAULT_MIMETYPE               "video/x-nv-yuv"
#define NVGST_DEFAULT_PREVIEW_MIMETYPE       "video/x-nv-yuv"
#define NVGST_DEFAULT_CAPTURE_MIMETYPE       "video/x-nv-yuv"
#define NVGST_DEFAULT_RENDER_TARGET          RENDER_OVERLAY
#define NVGST_DEFAULT_FPS                    30
#define NVGST_DEFAULT_AUTOMATE_MODE          FALSE
#define NVGST_AUTOMATE_CAPTURE               FALSE
#define NVGST_KPI_MEASURE                    FALSE
#define NVGST_RAW_PREV_CAP                   FALSE
#define NVGST_RAW_PREVIEW                    FALSE
#define NVGST_RAW_CAPTURE                    FALSE
#define NVGST_DEFAULT_FILENAME               "nvgsttest"
#define MAX_PCAP_RES                         7
#define MAX_ICAP_RES                         13
#define MAX_VCAP_RES                         8

/* STRESS && STABILITY*/
#define NVGST_DEFAULT_ITERATION_COUNT        1
#define NVGST_DEFAULT_CAPTURE_GAP            2.5
#define NVGST_DEFAULT_RETURN_VALUE           0
#define NVGST_DEFAULT_TOGGLE_AUTOFOCUS       FALSE
#define NVGST_DEFAULT_TOGGLE_SENSOR          FALSE
#define NVGST_DEFAULT_TOGGLE_CAMERA_MODE     FALSE
#define NVGST_DEFAULT_ALL_RESOLUTION_IMAGES  FALSE
#define NVGST_DEFAULT_ALL_RESOLUTION_VIDEOS  FALSE
#define NUMBER_OF_RESOLUTIONS                7  //QCIF (176x144), QVGA ( 320x240), CIF (352 x 288 ) , VGA(640x480),
                                                //D1 (720x480) , 720p (1280 x 720) , 1080p(1920 x 1088)

/* USB cam */
#define NVGST_DEFAULT_USBCAM_TYPE            YUV
#define NVGST_DEFAULT_VIDCAP_DEVICE          "/dev/video0"
#define NVGST_DEFAULT_CAPTURE_FORMAT         1
#define NVGST_DEFAULT_USBCAM_IMAGE_ENCODER   2
#define NVGST_DEFAULT_USBCAM_VIDEO_ENCODER   FORMAT_H264_HW
#define NVGST_DEFAULT_USBCAM_PREVIEW_WIDTH   640
#define NVGST_DEFAULT_USBCAM_PREVIEW_HEIGHT  480

typedef enum
{
  PR_176x144 = 0,
  PR_320x240,
  PR_640x480,
  PR_768x432,
  PR_720x480,
  PR_1280x720,
  PR_1920x1080
} Prev_Res;

typedef enum
{
  IR_176x144 = 0,
  IR_320x240,
  IR_480x480,
  IR_640x480,
  IR_800x600,
  IR_720x480,
  IR_1280x720,
  IR_1280x960,
  IR_1600x1200,
  IR_2048x1536,
  IR_2240x1680,
  IR_2560x1920,
  IR_4096x3072
} Icap_Res;

typedef enum
{
  VR_176x144 = 0,
  VR_320x240,
  VR_352x288,
  VR_640x480,
  VR_704x576,
  VR_720x480,
  VR_1280x720,
  VR_1920x1080
} Vcap_Res;

typedef enum
{
  YUV = 0,
  MJPEG
} USBCam_type;

typedef enum
{
  RENDER_EGL_MIXER = 0,
  RENDER_EGL,
  RENDER_OVERLAY
} RenderType;

typedef enum
{
  VOICE_OFF = 0,
  VOICE_ON
} VoiceControl;

typedef enum
{
  FILE_MP4 = 0,
  FILE_3GP,
  FILE_AVI
} FileType;

typedef enum
{
  FORMAT_H264_HW = 0,
  FORMAT_VP8_HW,
  FORMAT_MPEG4_SW,
  FORMAT_H263_SW
} VideoEncFormatType;

typedef enum
{
  CAPTURE_NONE = 0,
  CAPTURE_IMAGE,
  CAPTURE_VIDEO
} CaptureType;

typedef enum
{
  FORMAT_NTSC = 0,
  FORMAT_PAL
} FormatType;

typedef enum
{
  FRAME_PROGRESSIVE = 0,
  FRAME_INTERLACED
} FrameType;

typedef struct NvGstVipParam_
{
  gint frametype;
  gint formattype;
  gboolean vip_enable;
  gint rendertarget;
} NvGstVipParam;

typedef struct NvGstCSIParam_
{
  gboolean csi_enable;
} NvGstCSIParam;

typedef struct NvGstUsbCamParam_
{
  gint usbcam_type;
  gint capture_format;
  gint usbcam_capcount;
  gint preview_width;
  gint preview_height;
  gchar *cap_dev_node;
  gulong handler_id;
  gboolean usbcam_enable;
  gboolean usbcam_record;
  gboolean use_testsrc;
  gchar *vidcap_device;
  GstElement *usbcam_capturefilter;
  GstElement *colorspace_conv;
  GstElement *usbcam_prev_queue;
  GstElement *usbcam_cap_queue;
  GstElement *usbcam_tee;
  GstElement *nv_vidconv;
  GstElement *encoder;
  GstElement *vsrc;
  GstElement *vsink;
} NvGstUsbCamParam;

typedef struct NvGstCameraParam_
{
  gint mode;
  gint sensorid;
  gint preview_width;
  gint preview_height;
  gint preview_time;
  gint capture_start_time;
  gint image_capture_width;
  gint image_capture_height;
  gint video_capture_width;
  gint video_capture_height;
  gint capture_time;
  gint quit_time;
  gint exposure;
  gint iso;
  gint contrast;
  gint metermode;
  gint flicker;
  gint brightness;
  gint saturation;
  gint hue;
  gfloat edgeenhancement;
  gint imagefilter;
  gint whitebalance;
  gint preview_rotate;
  gint capture_rotate;
  gint image_enc;
  gint video_enc;
  gint audio_enc;
  gint channels;
  gint sample_rate;
  gint image_quality;
  gint videnc_quality_lvl;
  guint bitrate;
  gint iframeinterval;
  gint delaycapture;
  gint stereomode;
  gint rc_mode;
  gboolean noaudio;
  gboolean novideo;
  gboolean send_new_seg;
  gboolean use_camerabin;
  gint file_type;
  gint Voice_Control;
  gint capture_count;
  gboolean all_flags;
  guint32 fps_rate;
  gint flash_mode;
  gint scene_mode;
  gfloat zoom;
  gfloat ev_comp;
  gint focus_mode;
  gint noise_reduction;
  GstElement *vsrc;
  GstElement *vsink;
  GstElement *camera;
  GstElement *audio_src;
  GstElement *video_sink;
  GstElement *capturefilter;
  GstElement *image_filter;
  GstElement *audiofilter;
  GstElement *iencoder;
  GstElement *vencoder;
  GstElement *h264_filter;
  GstElement *h264parse;
  GstElement *muxer;
  GstElement *aencoder;
  GstElement *colorspace_conv;
  GMutex *lock;
  GCond *cond;
  gchar *qp_range;
  gchar *svs;
  gchar *mimetype;
  gchar *preview_mimetype;
  gchar *capture_mimetype;
  gboolean automate;
  gboolean automate_capture;
  gboolean perf_measure;
  gboolean raw_prev_cap;
  gboolean raw_preview;
  gboolean raw_capture;
  GTimer *timer;
  /* STRESS && STABILITY */
  gint iteration_count;
  gint capture_gap;
  gint return_value;
  gboolean toggle_autofocus;
  gboolean toggle_sensor;
  gboolean toggle_mode;
  gboolean all_resolution_images;
  gboolean all_resolution_videos;
  gchar *file_name;
  gchar *extra_options;
} NvGstCameraParam;

static NvGstCameraParam nvgstcameraParam =
    { NVGST_DEFAULT_CAPTURE_MODE, NVGST_DEFAULT_SENSOR_ID,
  NVGST_DEFAULT_PREVIEW_WIDTH, NVGST_DEFAULT_PREVIEW_HEIGHT,
  NVGST_DEFAULT_PREVIEW_TIME, NVGST_DEFAULT_CAP_START_DELAY,
  NVGST_DEFAULT_CAPTURE_WIDTH, NVGST_DEFAULT_CAPTURE_HEIGHT,
  NVGST_DEFAULT_CAPTURE_WIDTH, NVGST_DEFAULT_CAPTURE_HEIGHT,
  NVGST_DEFAULT_CAPTURE_TIME, NVGST_DEFAULT_QUIT_TIME, NVGST_DEFAULT_EXPOSURE,
  NVGST_DEFAULT_ISO,
  NVGST_DEFAULT_CONTRAST, NVGST_DEFAULT_METERMODE, NVGST_DEFAULT_FLICKER,
  NVGST_DEFAULT_BRIGHTNESS, NVGST_DEFAULT_SATURATION, NVGST_DEFAULT_HUE,
  NVGST_DEFAULT_EDGEENHANCEMENT, NVGST_DEFAULT_FILTER,
  NVGST_DEFAULT_WHITEBALANCE,
  NVGST_DEFAULT_PREVIEW_ROTATION, NVGST_DEFAULT_CAPTURE_ROTATION,
  NVGST_DEFAULT_IMAGE_ENCODER,
  NVGST_DEFAULT_VIDEO_ENCODER, NVGST_DEFAULT_AUDIO_ENCODER,
  NVGST_DEFAULT_CHANNELS,
  NVGST_DEFAULT_SAMPLE_RATE, NVGST_DEFAULT_IMAGE_QUALITY,
  NVGST_DEFAULT_VIDEO_QUALITY_LEVEL,
  NVGST_DEFAULT_BITRATE, NVGST_DEFAULT_IFRAMEINTERVAL,
  NVGST_DEFAULT_CAPTURE_DELAY,
  NVGST_DEFAULT_STEREO_MODE, NVGST_DEFAULT_RC_MODE, FALSE, FALSE, FALSE, FALSE,
  NVGST_DEFAULT_FILE_TYPE, NVGST_DEFAULT_VOICE_CONTROL,
  NVGST_DEFAULT_CAPTURE_COUNT, NVGST_SET_FLAGS, NVGST_DEFAULT_FPS,
  NVGST_DEFAULT_FLASH_MODE,
  NVGST_DEFAULT_SCENE_MODE, NVGST_DEFAULT_ZOOM, NVGST_DEFAULT_EVCOMP,
  NVGST_DEFAULT_FOCUS_MODE,
  NVGST_DEFAULT_NR, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  NVGST_DEFAULT_PREVIEW_MODE, NVGST_DEFAULT_MIMETYPE,
  NVGST_DEFAULT_PREVIEW_MIMETYPE, NVGST_DEFAULT_CAPTURE_MIMETYPE,
  NVGST_DEFAULT_AUTOMATE_MODE, NVGST_AUTOMATE_CAPTURE, NVGST_KPI_MEASURE,
  NVGST_RAW_PREV_CAP, NVGST_RAW_PREVIEW,
  NVGST_RAW_CAPTURE, NULL, NVGST_DEFAULT_ITERATION_COUNT,
  NVGST_DEFAULT_CAPTURE_GAP, NVGST_DEFAULT_RETURN_VALUE,
  NVGST_DEFAULT_TOGGLE_AUTOFOCUS, NVGST_DEFAULT_TOGGLE_SENSOR,
  NVGST_DEFAULT_TOGGLE_CAMERA_MODE, NVGST_DEFAULT_ALL_RESOLUTION_IMAGES,
  NVGST_DEFAULT_ALL_RESOLUTION_VIDEOS, NULL, NULL
};

static NvGstVipParam nvgstVipParam =
    { FRAME_INTERLACED, FORMAT_NTSC, FALSE, NVGST_DEFAULT_RENDER_TARGET };
static NvGstCSIParam nvgstCSIParam = { FALSE };

static NvGstUsbCamParam nvgstUsbCamParam =
    { NVGST_DEFAULT_USBCAM_TYPE, NVGST_DEFAULT_CAPTURE_FORMAT, 0,
  NVGST_DEFAULT_USBCAM_PREVIEW_WIDTH, NVGST_DEFAULT_USBCAM_PREVIEW_HEIGHT,
  NULL, 0,
  FALSE, FALSE, FALSE, NULL,
  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
};

static gint prevres_width[MAX_PCAP_RES] =
    { 176, 320, 640, 768, 720, 1280, 1920 };
static gint prevres_height[MAX_PCAP_RES] =
    { 144, 240, 480, 432, 480, 720, 1080 };
static gint imgres_width[MAX_ICAP_RES] =
    { 176, 320, 480, 640, 800, 720, 1280, 1280, 1600, 2048, 2240, 2560, 4096 };
static gint imgres_height[MAX_ICAP_RES] =
    { 144, 240, 480, 480, 600, 480, 720, 960, 1200, 1536, 1680, 1920, 3072 };
static gint vidres_width[MAX_VCAP_RES] =
    { 176, 320, 352, 640, 704, 720, 1280, 1920 };
static gint vidres_height[MAX_VCAP_RES] =
    { 144, 240, 188, 480, 576, 480, 720, 1080 };
static gboolean recording = FALSE;
static GMainLoop *loop;
static gboolean cintr = FALSE;
static gboolean create_pipeline (void);
static gboolean parse_spec (const gchar * option_name, const gchar * value,
    gpointer data, GError ** error);
static void destroy_pipeline (void);
static gboolean create_usbcam_pipeline (void);
static gboolean create_CSIcam_pipeline (void);
static void destroy_usbcam_pipeline (void);
static void destroy_CSIcam_pipeline (void);
static void restart_usbcam_native_pipeline (void);
static void print_help (void);
static gboolean
encoder_buf_prob (GstPad * pad, GstBuffer * buffer, gpointer u_data);
void set_new_video_file_name (int type);
void set_new_image_file_name (int type);
void set_capture_device_node (void);
static gboolean get_preview_resolution (gint res);
static gboolean get_imgcap_resolution (gint res);
static gboolean get_vidcap_resolution (gint res);

#ifdef USE_CAMERABIN2
static GstEncodingProfile *create_video_profile (gint muxer_type,
    gint video_enc, gint audio_enc);
static GstEncodingProfile *create_image_profile (gint encoder_type);
#endif

GOptionEntry entries[] = {
  {"prev_res", 'W', 0, G_OPTION_ARG_CALLBACK, parse_spec,
      "Preview width & height eg. -W 3", NULL},
  {"vcap_res", 'V', 0, G_OPTION_ARG_CALLBACK, parse_spec,
      "Video Capture width & height eg. -V 3", NULL},
  {"icap_res", 'I', 0, G_OPTION_ARG_CALLBACK, parse_spec,
      "Image Capture width & height eg. -I 3", NULL},
  {"mode", 'm', 0, G_OPTION_ARG_INT, &nvgstcameraParam.mode,
      "Capture mode value (2=video 1=still)", NULL},
  {"starttime", 'S', 0, G_OPTION_ARG_INT, &nvgstcameraParam.capture_start_time,
        "Start capture after specified time in seconds(use with --automate or -A only)",
      NULL},
  {"recordfor", 'T', 0, G_OPTION_ARG_INT, &nvgstcameraParam.capture_time,
        "Capture video for specified time in seconds(use with --automate or -A only)",
      NULL},
  {"quitafter", 'Q', 0, G_OPTION_ARG_INT, &nvgstcameraParam.quit_time,
        "Quit application once capture done after specified time in seconds(use with --automate or -A only)",
      NULL},
#ifdef CSI_CAPTURE_SUPPORTED
  {"exposure", 'e', 0, G_OPTION_ARG_INT, &nvgstcameraParam.exposure,
      "Capture exposure value ", NULL},
  {"iso", 'i', 0, G_OPTION_ARG_INT, &nvgstcameraParam.iso,
      "Capture iso value(should not be changed with auto exposure on)", NULL},
  {"contrast", 'n', 0, G_OPTION_ARG_INT, &nvgstcameraParam.contrast,
      "Capture contrast value", NULL},
  {"metermode", 'o', 0, G_OPTION_ARG_INT, &nvgstcameraParam.metermode,
        "Capture metermode value(should not be changed with auto exposure on)",
      NULL},
  {"flicker", 'f', 0, G_OPTION_ARG_INT, &nvgstcameraParam.flicker,
      "Capture flicker value", NULL},
  {"brightness", 'b', 0, G_OPTION_ARG_INT, &nvgstcameraParam.brightness,
      "Capture brightness value", NULL},
  {"saturation", 's', 0, G_OPTION_ARG_INT, &nvgstcameraParam.saturation,
      "Capture saturation value", NULL},
  {"hue", 'h', 0, G_OPTION_ARG_INT, &nvgstcameraParam.hue, "Capture hue value",
      NULL},
  {"edgeenhancement", 'y', 0, G_OPTION_ARG_CALLBACK, parse_spec,
      "Capture edgeenhancement value", NULL},
  {"imagefilter", 'g', 0, G_OPTION_ARG_INT, &nvgstcameraParam.imagefilter,
      "Capture imagefilter value", NULL},
  {"whitebalance", 'w', 0, G_OPTION_ARG_INT, &nvgstcameraParam.whitebalance,
      "Capture whitebalance value", NULL},
  {"stereo", 'd', 0, G_OPTION_ARG_INT, &nvgstcameraParam.stereomode,
      "Stereo Mode (0=Off 1=Left Only 2=Right Only 3=Stereo full)", NULL},
  {"audio_enc", 'u', 0, G_OPTION_ARG_INT, &nvgstcameraParam.audio_enc,
      "Audio encode type (0=aac 1=amr)", NULL},
  {"voice_control", 'x', 0, G_OPTION_ARG_INT, &nvgstcameraParam.Voice_Control,
      "Voice control value (1=on 0=off)", NULL},
  {"sensorid", 'q', 0, G_OPTION_ARG_INT, &nvgstcameraParam.sensorid,
        "Camera Sensor id (0=Primary(Left-back) 1=Secondary(Front) 3=stereo)",
      NULL},
  {"fl_mode", 'F', 0, G_OPTION_ARG_INT, &nvgstcameraParam.flash_mode,
      "Camera Flash Mode", NULL},
  {"zoom", 'Z', 0, G_OPTION_ARG_CALLBACK, parse_spec, "Camera zoom level",
      NULL},
  {"csi", 0, 0, G_OPTION_ARG_NONE, &nvgstCSIParam.csi_enable,
      "Use VI-CSI input", NULL},
  {"rt", 0, 0, G_OPTION_ARG_INT, &nvgstVipParam.rendertarget,
      "Render Target, 0=(Mixer + EGL), 1=(EGL), 2=(Mixer + Overlay)", NULL},
#endif
  {"image_enc", 'J', 0, G_OPTION_ARG_INT, &nvgstcameraParam.image_enc,
        "Image encoder type (0=jpeg_HW[nv_omx_jpegenc] 1=jpeg_SW[jpegenc] 2=jpeg_HW[nvjpegenc])",
      NULL},
  {"video_enc", 'v', 0, G_OPTION_ARG_INT, &nvgstcameraParam.video_enc,
        "Video encoder type (0=h264[nv_omx_h264enc] 1=vp8[nv_omx_vp8enc] 2=mpeg4_SW[ffenc_mpeg4]) 3=H263_SW[ffenc_h263][Set Default Resolution:704x576(4CIF)]",
      NULL},
  // { "image_quality", 'l', 0, G_OPTION_ARG_INT, &nvgstcameraParam.image_quality, "Image quality value", NULL },
  // { "videnc_quality_lvl", 't', 0, G_OPTION_ARG_INT, &nvgstcameraParam.videnc_quality_lvl, "Video encoder quality level", NULL },
  // { "bitrate", 'z', 0, G_OPTION_ARG_INT, &nvgstcameraParam.bitrate, "Encode bitrate value", NULL },
  // { "iframeinterval", 'j', 0, G_OPTION_ARG_INT, &nvgstcameraParam.iframeinterval, "Encode iframeinterval value", NULL },
  // { "channels", 'a', 0, G_OPTION_ARG_INT, &nvgstcameraParam.channels, "Audio channels value", NULL },
  // { "sample_rate", 'r', 0, G_OPTION_ARG_INT, &nvgstcameraParam.sample_rate, "Audio sample-rate value", NULL },
  {"file_type", 'k', 0, G_OPTION_ARG_INT, &nvgstcameraParam.file_type,
      "Container file type (0=mp4 1=3gp 2=avi)", NULL},
  //  { "sc_mode", 'S', 0, G_OPTION_ARG_INT, &nvgstcameraParam.scene_mode, "Camera Scene Mode", NULL },
  // { "ev_comp", 'E', 0, G_OPTION_ARG_DOUBLE, &nvgstcameraParam.ev_comp, "Camera ev-compensation", NULL },
  // { "fo_mode", 'C', 0, G_OPTION_ARG_INT, &nvgstcameraParam.focus_mode, "Camera Focus Mode", NULL },
  // { "nr", 'N', 0, G_OPTION_ARG_INT, &nvgstcameraParam.noise_reduction, "Camera noise reduction mode", NULL },
  {"frate", 0, 0, G_OPTION_ARG_INT, &nvgstcameraParam.fps_rate,
      "Capture Frame Rate (min = 5, max = 30)", NULL},
#ifdef USE_CAMERABIN2
  {"use-camerabin", 0, 0, G_OPTION_ARG_NONE, &nvgstcameraParam.use_camerabin,
      "Use Camerabin2", NULL},
#endif
  {"svs", 0, 0, G_OPTION_ARG_CALLBACK, parse_spec,
      "(=) chain for video Preview", NULL},
  /*
     {"qp-range", 0, 0, G_OPTION_ARG_CALLBACK, parse_spec,
     "Quantization range for P and I frame respectively e.g --qp-range=\"10-51:5-30\"",
     NULL},
   */
  {"automate", 'A', 0, G_OPTION_ARG_NONE, &nvgstcameraParam.automate,
      "Run application in automation mode", NULL},
#ifdef GST_KPI_MEASURE
  {"perf", 'P', 0, G_OPTION_ARG_NONE, &nvgstcameraParam.perf_measure,
      "Enable Camera Performance index measurement", NULL},
#endif
  {"raw", 'R', 0, G_OPTION_ARG_NONE, &nvgstcameraParam.raw_prev_cap,
      "Enable Camera raw-yuv both Preview-Capture", NULL},
  {"raw-preview", 'E', 0, G_OPTION_ARG_NONE, &nvgstcameraParam.raw_preview,
      "Enable Camera only raw-yuv Preview", NULL},
  {"raw-capture", 'U', 0, G_OPTION_ARG_NONE, &nvgstcameraParam.raw_capture,
      "Enable Camera only raw-yuv Capture", NULL},
  {"capture_auto", 'C', 0, G_OPTION_ARG_NONE,
        &nvgstcameraParam.automate_capture,
      "Do image/video capture in automation mode", NULL},
  {"vip", 0, 0, G_OPTION_ARG_NONE, &nvgstVipParam.vip_enable, "Use Video input",
      NULL},
  {"format_type", 0, 0, G_OPTION_ARG_INT, &nvgstVipParam.formattype,
      "Capture format type, 0=NTSC, 1=PAL", NULL},
  {"frame_type", 0, 0, G_OPTION_ARG_INT, &nvgstVipParam.frametype,
      "Capture frame type, 0=Progressive, 1=Interlaced", NULL},
  {"usbcam", 0, 0, G_OPTION_ARG_NONE, &nvgstUsbCamParam.usbcam_enable,
      "Use USB Camera input", NULL},
  {"usbcam_type", 'H', 0, G_OPTION_ARG_INT, &nvgstUsbCamParam.usbcam_type,
      "USB Camera type, 0=YUV, 1=MJPEG", NULL},
  {"testsrc", 0, 0, G_OPTION_ARG_NONE, &nvgstUsbCamParam.use_testsrc,
      "Use video test source input", NULL},
  {"cap_format", 'M', 0, G_OPTION_ARG_INT, &nvgstUsbCamParam.capture_format,
      "Capture Format, 0=I420, 1=YV12, 2=NV12, 3=YUY2, 4=UYVY, 5=YVYU, 6=Y444(Use --usbcam for cap format 0,1,3 & --testsrc for cap format 2,4,5,6) ",
      NULL},
  {"cap-dev-node", 0, 0, G_OPTION_ARG_CALLBACK, parse_spec,
      "Video capture device node (0=/dev/video0[default], 1=/dev/video1, 2=/dev/video2) "
          "e.g., --cap-dev-node=0", NULL},
  /* STRESS && STABILITY */
  {"count", 0, 0, G_OPTION_ARG_INT, &nvgstcameraParam.iteration_count,
      "Number of Iterations/Counts of stress testcase", NULL},
  {"capture-gap", 0, 0, G_OPTION_ARG_INT, &nvgstcameraParam.capture_gap,
        "Number of Seconds between successive image/video capture (only with '--automate') ",
      NULL},
  {"toggle-autofocus", 0, 0, G_OPTION_ARG_NONE,
        &nvgstcameraParam.toggle_autofocus,
        "Toggle AutoFocus for Count number of times (default = 1) (only with '--automate')",
      NULL},
  {"toggle-sensor", 0, 0, G_OPTION_ARG_NONE, &nvgstcameraParam.toggle_sensor,
        "Toggle Camera Sensor between Primary (Back-Left) and Secondary (Front) for Count number of times (default = 0) (only with '--automate')",
      NULL},
  {"toggle-mode", 0, 0, G_OPTION_ARG_NONE, &nvgstcameraParam.toggle_mode,
        "Toggle Between Still and Video Capture Modes for Count number of times (default = 1) (only with '--automate')",
      NULL},
  {"all-resolution-images", 0, 0, G_OPTION_ARG_NONE,
        &nvgstcameraParam.all_resolution_images,
        "Capture Images of All Resolutions (do not use 'count' in this case) (only with '--automate')",
      NULL},
  {"all-resolution-videos", 0, 0, G_OPTION_ARG_NONE,
        &nvgstcameraParam.all_resolution_videos,
        "Record Videos of All Resolutions (do not use 'count' in this case) (only with '--automate')",
      NULL},
  {"file-name", 0, 0, G_OPTION_ARG_STRING, &nvgstcameraParam.file_name,
        "File name to be used for images or videos. nvgsttest is used by default",
      NULL},
  {NULL}
};

static gboolean
parse_spec (const gchar * option_name, const gchar * value,
    gpointer data, GError ** error)
{
  if ((!g_strcmp0 ("--icap_res", option_name))
      || (!g_strcmp0 ("-I", option_name))) {
    if (TRUE != get_imgcap_resolution (atoi (value)))
      return FALSE;
  } else if ((!g_strcmp0 ("--vcap_res", option_name))
      || (!g_strcmp0 ("-V", option_name))) {
    if (TRUE != get_vidcap_resolution (atoi (value)))
      return FALSE;
  } else if ((!g_strcmp0 ("--prev_res", option_name))
      || (!g_strcmp0 ("-W", option_name))) {
    if (TRUE != get_preview_resolution (atoi (value)))
      return FALSE;
#ifdef CSI_CAPTURE_SUPPORTED
  } else if ((!g_strcmp0 ("--edgeenhancement", option_name))
      || (!g_strcmp0 ("-y", option_name))) {
    nvgstcameraParam.edgeenhancement = atof (value);
  } else if ((!g_strcmp0 ("--zoom", option_name))
      || (!g_strcmp0 ("-Z", option_name))) {
    nvgstcameraParam.zoom = atof (value);
#endif
  } else if (!g_strcmp0 ("--svs", option_name)) {
    nvgstcameraParam.svs = g_strdup (value);
    if (!g_strcmp0 ("xvimagesink", nvgstcameraParam.svs))
      nvgstcameraParam.preview_mimetype = "video/x-raw-yuv";
    else if (!g_strcmp0 ("nv_omx_videosink", nvgstcameraParam.svs))
      nvgstcameraParam.preview_mimetype = "video/x-nv-yuv";
    else if (!g_strcmp0 ("nv_omx_hdmi_videosink", nvgstcameraParam.svs))
      nvgstcameraParam.preview_mimetype = "video/x-nv-yuv";
    else if (!g_strcmp0 ("ximagesink", nvgstcameraParam.svs))
      nvgstcameraParam.preview_mimetype = "video/x-raw-yuv";
  } else if (!g_strcmp0 ("--qp-range", option_name)) {
    nvgstcameraParam.qp_range = g_strdup (value);
  } else if (!g_strcmp0 ("--cap-dev-node", option_name)) {
    if (nvgstUsbCamParam.cap_dev_node) {
      g_free (nvgstUsbCamParam.cap_dev_node);
      nvgstUsbCamParam.cap_dev_node = NULL;
    }

    nvgstUsbCamParam.cap_dev_node = g_strdup (value);
    set_capture_device_node ();
  }

  return TRUE;
}

void
set_capture_device_node (void)
{
  if (nvgstUsbCamParam.vidcap_device) {
    g_free (nvgstUsbCamParam.vidcap_device);
    nvgstUsbCamParam.vidcap_device = NULL;
  }

  nvgstUsbCamParam.vidcap_device = g_strndup ("/dev/video", 12);
  nvgstUsbCamParam.vidcap_device =
      strcat (nvgstUsbCamParam.vidcap_device,
      nvgstUsbCamParam.cap_dev_node);
}


static gboolean
get_preview_resolution (gint res)
{
  gboolean ret = TRUE;

  if ((res < PR_176x144) || (res > PR_1920x1080)) {
    g_print ("Invalid preview resolution\n");
    return FALSE;
  }

  nvgstcameraParam.preview_width = prevres_width[res];
  nvgstcameraParam.preview_height = prevres_height[res];

  return ret;
}

static gboolean
get_imgcap_resolution (gint res)
{
  gboolean ret = TRUE;

  if ((res < IR_176x144) || (res > IR_4096x3072)) {
    g_print ("Invalid image capture resolution\n");
    return FALSE;
  }

  nvgstcameraParam.image_capture_width = imgres_width[res];
  nvgstcameraParam.image_capture_height = imgres_height[res];

  return ret;
}

static gboolean
get_vidcap_resolution (gint res)
{
  gboolean ret = TRUE;

  if ((res < VR_176x144) || (res > VR_1920x1080)) {
    g_print ("Invalid video capture resolution\n");
    return FALSE;
  }

  nvgstcameraParam.video_capture_width = vidres_width[res];
  nvgstcameraParam.video_capture_height = vidres_height[res];

  return ret;
}

static gboolean
data_probe (GstPad * pad, GstMiniObject * obj, gpointer data)
{
  gboolean ret = TRUE;

  if (GST_IS_BUFFER (obj)) {
    if (G_UNLIKELY (nvgstcameraParam.send_new_seg)) {
      GstBuffer *buf = GST_BUFFER_CAST (obj);
      GstClockTime ts = GST_BUFFER_TIMESTAMP (buf);
      GstPad *peer;

      if (!GST_CLOCK_TIME_IS_VALID (ts)) {
        ts = 0;
      }
      peer = gst_pad_get_peer (pad);
      g_return_val_if_fail (peer != NULL, TRUE);

      gst_pad_send_event (peer, gst_event_new_new_segment (FALSE, 1.0,
              GST_FORMAT_TIME, ts, -1, 0));

      gst_object_unref (peer);
      nvgstcameraParam.send_new_seg = FALSE;
    }
  }

  return ret;
}


static gpointer
reset_elements (gpointer data)
{
  if (nvgstcameraParam.mode == CAPTURE_VIDEO) {
    gst_element_set_state (nvgstcameraParam.capturefilter, GST_STATE_READY);
    gst_element_set_state (nvgstcameraParam.vencoder, GST_STATE_READY);
    gst_element_set_state (nvgstcameraParam.muxer, GST_STATE_READY);

    gst_element_sync_state_with_parent (nvgstcameraParam.capturefilter);
    gst_element_sync_state_with_parent (nvgstcameraParam.vencoder);

    if (nvgstcameraParam.h264_filter) {
      gst_element_set_state (nvgstcameraParam.h264_filter, GST_STATE_READY);
      gst_element_sync_state_with_parent (nvgstcameraParam.h264_filter);
    }
    if (nvgstcameraParam.h264parse) {
      gst_element_set_state (nvgstcameraParam.h264parse, GST_STATE_READY);
      gst_element_sync_state_with_parent (nvgstcameraParam.h264parse);
    }

    gst_element_sync_state_with_parent (nvgstcameraParam.muxer);

    if (nvgstcameraParam.audio_src) {
      gst_element_set_state (nvgstcameraParam.audiofilter, GST_STATE_READY);
      gst_element_set_state (nvgstcameraParam.audio_src, GST_STATE_NULL);
      gst_element_set_state (nvgstcameraParam.aencoder, GST_STATE_READY);
      gst_element_sync_state_with_parent (nvgstcameraParam.audiofilter);
      gst_element_sync_state_with_parent (nvgstcameraParam.aencoder);
    }
  } else if (nvgstcameraParam.mode == CAPTURE_IMAGE) {
    gst_element_set_state (nvgstcameraParam.image_filter, GST_STATE_READY);
    gst_element_sync_state_with_parent (nvgstcameraParam.image_filter);

    gst_element_set_state (nvgstcameraParam.iencoder, GST_STATE_NULL);
    gst_element_sync_state_with_parent (nvgstcameraParam.iencoder);
  }
  return NULL;
}

static gboolean
bus_call (GstBus * bus, GstMessage * msg, gpointer data)
{
  switch (GST_MESSAGE_TYPE (msg)) {
    case GST_MESSAGE_ERROR:{
      GError *err = NULL;
      gchar *name, *debug = NULL;

      name = gst_object_get_path_string (msg->src);
      gst_message_parse_error (msg, &err, &debug);

      g_printerr ("ERROR on bus: by %s: %s\n", name, err->message);
      if (debug != NULL)
        g_printerr ("debug info:\n%s\n", debug);

      g_error_free (err);
      g_free (debug);
      g_free (name);

      nvgstcameraParam.return_value = -1;
      g_main_loop_quit (loop);

    }
      break;

    case GST_MESSAGE_STATE_CHANGED:{
      GstState old, new, pending;

      gst_message_parse_state_changed (msg, &old, &new, &pending);

      GST_DEBUG_OBJECT (GST_OBJECT (msg->src),
          "changed state from %s to %s, pending %s\n",
          gst_element_state_get_name (old), gst_element_state_get_name (new),
          gst_element_state_get_name (pending));
      if (GST_MESSAGE_SRC (msg) == GST_OBJECT (nvgstcameraParam.camera)
          && pending == GST_STATE_VOID_PENDING && old == GST_STATE_PAUSED
          && new == GST_STATE_PLAYING) {
      }
    }
      break;

    case GST_MESSAGE_EOS:{
      if ((nvgstcameraParam.mode == CAPTURE_VIDEO) &&
          (nvgstUsbCamParam.usbcam_enable || nvgstUsbCamParam.use_testsrc
              || nvgstCSIParam.csi_enable)) {
        restart_usbcam_native_pipeline ();
      }

      if (nvgstcameraParam.mode == CAPTURE_IMAGE) {
        gboolean ret = FALSE;
        g_mutex_lock (nvgstcameraParam.lock);

        recording = FALSE;
        g_cond_signal (nvgstcameraParam.cond);
        g_mutex_unlock (nvgstcameraParam.lock);

        if (nvgstcameraParam.sensorid != 1 &&
            !(nvgstUsbCamParam.usbcam_enable ||
                nvgstUsbCamParam.use_testsrc || nvgstCSIParam.csi_enable)) {

          g_signal_emit_by_name (G_OBJECT (nvgstcameraParam.vsrc),
              "stop-auto-focus", &ret);
        }
      }
    }
      break;

    case GST_MESSAGE_APPLICATION:{
      const GstStructure *s;
      s = gst_message_get_structure (msg);

      if (gst_structure_has_name (s, "NvGstAppInterrupt")) {
        g_print ("Terminating the pipeline ...\n");
        g_main_loop_quit (loop);
      }
    }
      break;
    case GST_MESSAGE_ELEMENT:{
      const GstStructure *s;
      s = gst_message_get_structure (msg);

      if (gst_structure_has_name (s, "paused-after-capture")) {
        g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "viewfinder", TRUE,
            NULL);
      }
    }
      break;
    default:
      break;
  }

  return TRUE;
}


static gboolean
on_input (GIOChannel * ichannel, GIOCondition cond, gpointer data)
{
  static gchar buf[256];
  int bytes_read;
  gint fd;
  gboolean ret = FALSE;
  gint val;
  gfloat fval;
  gchar *arr = NULL;

  fd = g_io_channel_unix_get_fd (ichannel);
  bytes_read = read (fd, buf, 256);
  buf[bytes_read - 1] = '\0';


  if (g_str_has_prefix (buf, "-h")) {
    print_help ();

  } else if (buf[0] == 'q') {
    g_main_loop_quit (loop);

  } else if (buf[0] == '1' && nvgstcameraParam.mode == CAPTURE_VIDEO
      && recording == FALSE) {
    set_new_video_file_name (nvgstcameraParam.file_type);
    recording = TRUE;
    if (nvgstUsbCamParam.usbcam_enable ||
        nvgstUsbCamParam.use_testsrc || nvgstCSIParam.csi_enable) {
      nvgstUsbCamParam.usbcam_record = TRUE;
      if (nvgstcameraParam.use_camerabin)
        g_signal_emit_by_name (G_OBJECT (nvgstcameraParam.camera),
            "start-capture");

    } else {
      if (nvgstcameraParam.use_camerabin) {
        g_signal_emit_by_name (G_OBJECT (nvgstcameraParam.camera),
            "start-capture", &ret);
      } else {
        if (nvgstcameraParam.audio_src) {
          GstClock *clock =
              gst_pipeline_get_clock (GST_PIPELINE_CAST
              (nvgstcameraParam.camera));

          gst_element_set_state (nvgstcameraParam.audio_src, GST_STATE_PAUSED);
          gst_element_set_base_time (nvgstcameraParam.audio_src,
              gst_element_get_base_time (GST_ELEMENT_CAST
                  (nvgstcameraParam.camera)));
          if (clock) {
            gst_element_set_clock (nvgstcameraParam.audio_src, clock);
            gst_object_unref (clock);
          }
        }
        g_signal_emit_by_name (G_OBJECT (nvgstcameraParam.vsrc),
            "start-capture", &ret);
        if (nvgstcameraParam.audio_src)
          gst_element_set_locked_state (nvgstcameraParam.audio_src, FALSE);

        nvgstcameraParam.send_new_seg = TRUE;
        if (nvgstcameraParam.audio_src)
          gst_element_set_state (nvgstcameraParam.audio_src, GST_STATE_PLAYING);
      }
    }
    g_print ("\nSet To Recording, Enter (0) to stop\n");

  } else if (buf[0] == '0' && recording == TRUE) {
    recording = FALSE;
    if (nvgstUsbCamParam.usbcam_enable ||
        nvgstUsbCamParam.use_testsrc || nvgstCSIParam.csi_enable) {
      nvgstUsbCamParam.usbcam_record = FALSE;
      if (nvgstcameraParam.use_camerabin) {
        g_signal_emit_by_name (G_OBJECT (nvgstcameraParam.camera),
            "stop-capture");
      } else {
        gst_pad_send_event (gst_element_get_static_pad
            (nvgstUsbCamParam.usbcam_cap_queue, "sink"), gst_event_new_eos ());
        gst_pad_send_event (gst_element_get_static_pad (nvgstUsbCamParam.vsink,
                "sink"), gst_event_new_eos ());
      }
    } else {
      if (nvgstcameraParam.use_camerabin) {
        g_signal_emit_by_name (G_OBJECT (nvgstcameraParam.camera),
            "stop-capture", &ret);
      } else {
        g_signal_emit_by_name (G_OBJECT (nvgstcameraParam.vsrc), "stop-capture",
            &ret);
        if (VOICE_OFF != nvgstcameraParam.Voice_Control) {
          gst_element_send_event (nvgstcameraParam.audio_src,
              gst_event_new_eos ());
        }
        sleep (1);
        if (!g_thread_create (reset_elements, NULL, FALSE, NULL)) {
          nvgstcameraParam.return_value = -1;
          g_print ("Failed to create thread to "
              "reset video elements' state, video recordings may not work "
              "anymore");
        }
      }
    }
    g_print ("Stopped Recording \n");
  } else if (!g_strcmp0 (buf, "gqp") && recording == TRUE) {

    if (nvgstUsbCamParam.usbcam_enable ||
        nvgstUsbCamParam.use_testsrc || nvgstCSIParam.csi_enable) {
      g_object_get (G_OBJECT (nvgstUsbCamParam.encoder), "qp-range", &arr,
          NULL);
    } else {
      g_object_get (G_OBJECT (nvgstcameraParam.vencoder), "qp-range", &arr,
          NULL);
    }
    g_print ("val = %s\n", arr);
    if (arr)
      g_free (arr);

  } else if (buf[0] == 'j' && nvgstcameraParam.mode == CAPTURE_IMAGE
      && recording == FALSE) {
    gint n = 0;
    gint count = 1;
    gchar *str;
    gint stime = 0;

    str = g_strrstr (buf, ":");
    if (str) {
      count = atoi (str + 1);
    }

    str = g_strrstr (buf, "x");
    if (str) {
      stime = atoi (str + 1);
      if (stime < 500)
        stime = 500;
      g_usleep (stime * 1000 - 500000);
    }

    if (!(nvgstUsbCamParam.usbcam_enable ||
            nvgstUsbCamParam.use_testsrc || nvgstCSIParam.csi_enable)) {

      if (count > 1)
        g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "pause-after-capture",
            FALSE, NULL);
      else
        g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "pause-after-capture",
            TRUE, NULL);

      if (nvgstcameraParam.sensorid != 1) {
        g_signal_emit_by_name (G_OBJECT (nvgstcameraParam.vsrc),
            "start-auto-focus", TRUE, TRUE, TRUE, &ret);
      }

      while (n++ < count) {
        recording = TRUE;

        if (nvgstcameraParam.use_camerabin) {
          set_new_image_file_name (0);
          g_signal_emit_by_name (G_OBJECT (nvgstcameraParam.camera),
              "start-capture", &ret);
        } else
          g_signal_emit_by_name (G_OBJECT (nvgstcameraParam.vsrc),
              "start-capture", &ret);

        g_mutex_lock (nvgstcameraParam.lock);
        while (recording) {
          g_cond_wait (nvgstcameraParam.cond, nvgstcameraParam.lock);
        }

        g_mutex_unlock (nvgstcameraParam.lock);

        g_print ("Image Captured \n");
        g_usleep (250000);
      }

      gst_element_post_message (GST_ELEMENT (nvgstcameraParam.camera),
          gst_message_new_eos (GST_OBJECT (nvgstcameraParam.camera)));

    } else {
      while (n++ < count) {
        recording = TRUE;
        if (nvgstcameraParam.use_camerabin) {
          set_new_image_file_name (0);
          g_signal_emit_by_name (G_OBJECT (nvgstcameraParam.camera),
              "start-capture");
        } else {
          nvgstUsbCamParam.usbcam_capcount = 0;
          nvgstUsbCamParam.usbcam_record = TRUE;
        }

        g_mutex_lock (nvgstcameraParam.lock);
        while (recording) {
          g_cond_wait (nvgstcameraParam.cond, nvgstcameraParam.lock);
        }
        g_mutex_unlock (nvgstcameraParam.lock);
        g_print ("Image Captured from USB cam \n");
        g_usleep (250000);
      }
    }

    gst_element_post_message (GST_ELEMENT (nvgstcameraParam.camera),
        gst_message_new_eos (GST_OBJECT (nvgstcameraParam.camera)));

  } else if (buf[0] == 'f' && nvgstcameraParam.mode == CAPTURE_VIDEO
      && recording == TRUE) {

    g_print ("Forcing IDR on video encoder\n");
    g_signal_emit_by_name (G_OBJECT (nvgstcameraParam.vencoder), "force-IDR",
        &ret);

  } else if (g_str_has_prefix (buf, "tt:")
      && nvgstcameraParam.mode == CAPTURE_VIDEO && recording == TRUE) {
    val = atoi (buf + 3);
    if (val < 0 || val > 4) {
      val = NVGST_DEFAULT_TEMPORAL_TRADEOFF;
      g_print
          ("Invalid input value of temporal tradeoff, setting to default = 0 \n");
    }
    g_print ("val = %d\n", val);
    if (nvgstUsbCamParam.usbcam_enable ||
        nvgstUsbCamParam.use_testsrc || nvgstCSIParam.csi_enable) {
      g_object_set (G_OBJECT (nvgstUsbCamParam.encoder), "temporal-tradeoff",
          val, NULL);
    } else {
      g_object_set (G_OBJECT (nvgstcameraParam.vencoder), "temporal-tradeoff",
          val, NULL);
    }
  } else if (!g_strcmp0 (buf, "gtt")) {
    if (nvgstUsbCamParam.usbcam_enable ||
        nvgstUsbCamParam.use_testsrc || nvgstCSIParam.csi_enable) {
      g_object_get (G_OBJECT (nvgstUsbCamParam.encoder), "temporal-tradeoff",
          &val, NULL);
    } else {
      g_object_get (G_OBJECT (nvgstcameraParam.vencoder), "temporal-tradeoff",
          &val, NULL);
    }
    g_print ("val = %d\n", val);

  } else if (g_str_has_prefix (buf, "br:")) {
    fval = atof (buf + 3);
    nvgstcameraParam.bitrate = fval;
    if (nvgstUsbCamParam.usbcam_enable ||
        nvgstUsbCamParam.use_testsrc || nvgstCSIParam.csi_enable) {
      if (!nvgstUsbCamParam.encoder)
        g_print ("encoder null\n");
      g_print ("using\n");
      g_object_set (G_OBJECT (nvgstUsbCamParam.encoder), "bitrate",
          nvgstcameraParam.bitrate, NULL);
    } else {
      g_object_set (G_OBJECT (nvgstcameraParam.vencoder), "bitrate",
          nvgstcameraParam.bitrate, NULL);
    }
    g_print ("bitrate =: %u\n", nvgstcameraParam.bitrate);

  } else if (!g_strcmp0 (buf, "gbr")) {
    guint br;
    if (nvgstUsbCamParam.usbcam_enable ||
        nvgstUsbCamParam.use_testsrc || nvgstCSIParam.csi_enable) {
      g_object_get (G_OBJECT (nvgstUsbCamParam.encoder), "bitrate", &br, NULL);
    } else {
      g_object_get (G_OBJECT (nvgstcameraParam.vencoder), "bitrate", &br, NULL);
    }
    g_print ("bitrate =: %u\n", br);

  } else if (g_str_has_prefix (buf, "rc:")) {
    val = atoi (buf + 3);
    if (val < 0 || val > 2) {
      nvgstcameraParam.rc_mode = NVGST_DEFAULT_RC_MODE;
      g_print ("Invalid rc-mode, setting rc-mode to default val = 2(VBR2)\n");
    } else {
      nvgstcameraParam.rc_mode = val;
    }
    if (nvgstUsbCamParam.usbcam_enable ||
        nvgstUsbCamParam.use_testsrc || nvgstCSIParam.csi_enable) {
      g_object_set (G_OBJECT (nvgstUsbCamParam.encoder), "rc-mode",
          nvgstcameraParam.rc_mode, NULL);
    } else {
      g_object_set (G_OBJECT (nvgstcameraParam.vencoder), "rc-mode",
          nvgstcameraParam.rc_mode, NULL);
    }
    g_print ("rc-mode =: %d\n", nvgstcameraParam.rc_mode);

  } else if (!g_strcmp0 (buf, "grc")) {
    gint rc;
    if (nvgstUsbCamParam.usbcam_enable ||
        nvgstUsbCamParam.use_testsrc || nvgstCSIParam.csi_enable) {
      g_object_get (G_OBJECT (nvgstUsbCamParam.encoder), "rc-mode", &rc, NULL);
    } else {
      g_object_get (G_OBJECT (nvgstcameraParam.vencoder), "rc-mode", &rc, NULL);
    }
    g_print ("rc-mode =: %d\n", rc);

#ifdef CSI_CAPTURE_SUPPORTED
  } else if (buf[0] == 'z') {
    fval = atof (buf + 2);
    if (nvgstcameraParam.use_camerabin) {
      g_object_set (G_OBJECT (nvgstcameraParam.camera), "zoom", fval, NULL);
    } else {
      g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "zoom", fval, NULL);
    }
    g_print ("zoom = %f\n", fval);

  } else if (!g_strcmp0 (buf, "gz")) {
    if (nvgstcameraParam.use_camerabin) {
      g_object_get (G_OBJECT (nvgstcameraParam.camera), "zoom", &fval, NULL);
    } else {
      g_object_get (G_OBJECT (nvgstcameraParam.vsrc), "zoom", &fval, NULL);
    }
    g_print ("val =: %f\n", fval);
#endif

  } else if (recording == FALSE) {
    if (g_str_has_prefix (buf, "icr:")) {
      GstCaps *caps;

      if (TRUE != get_imgcap_resolution (atoi (buf + 4)))
        goto invalid_input;

      g_print ("w = %d, h = %d\n", nvgstcameraParam.image_capture_width,
          nvgstcameraParam.image_capture_height);

      if (nvgstUsbCamParam.usbcam_enable || nvgstUsbCamParam.use_testsrc
          || nvgstCSIParam.csi_enable) {
        restart_usbcam_native_pipeline ();
      } else {
        caps = gst_caps_new_simple (nvgstcameraParam.mimetype,
            "width", G_TYPE_INT, nvgstcameraParam.image_capture_width,
            "height", G_TYPE_INT, nvgstcameraParam.image_capture_height, NULL);

        if (nvgstcameraParam.use_camerabin) {
          g_object_set (G_OBJECT (nvgstcameraParam.camera),
              "image-capture-caps", caps, NULL);
        } else {
          //This is a WAR since openmax is not returning OMX_PortEnableDone event Bug 1353010
          //keeping old code below
          destroy_pipeline ();
          create_pipeline ();

          /*if(nvgstcameraParam.vsrc)
             pad = gst_element_get_static_pad (nvgstcameraParam.vsrc, "imgsrc");
             g_object_set(G_OBJECT(nvgstcameraParam.image_filter), "caps", caps, NULL);
             if(pad){
             gst_pad_send_event (pad, gst_event_new_custom (GST_EVENT_CUSTOM_BOTH,
             gst_structure_new ("renegotiate", NULL)));
             gst_object_unref (pad);
             }
             sleep(1);
             if (!g_thread_create (reset_elements,
             NULL, FALSE, NULL)) {
             nvgstcameraParam.return_value = -1;
             g_print( "Failed to create thread to "
             "reset image encode elements' state, image encoding may not work "
             "anymore");
             } */
        }
        gst_caps_unref (caps);
      }
    } else if (g_str_has_prefix (buf, "vcr:")) {
      GstCaps *caps;

      if (TRUE != get_vidcap_resolution (atoi (buf + 4)))
        goto invalid_input;

      g_print ("w = %d, h = %d\n", nvgstcameraParam.video_capture_width,
          nvgstcameraParam.video_capture_height);

      if (nvgstUsbCamParam.usbcam_enable || nvgstUsbCamParam.use_testsrc
          || nvgstCSIParam.csi_enable) {
        restart_usbcam_native_pipeline ();
      } else {
        caps = gst_caps_new_simple (nvgstcameraParam.mimetype,
            "width", G_TYPE_INT, nvgstcameraParam.video_capture_width,
            "height", G_TYPE_INT, nvgstcameraParam.video_capture_height, NULL);

        if (nvgstcameraParam.use_camerabin) {
          g_object_set (G_OBJECT (nvgstcameraParam.camera),
              "video-capture-caps", caps, NULL);
        } else {
          //This is a WAR since openmax is not returning OMX_PortEnableDone event Bug 1353010
          //keeping old code below
          destroy_pipeline ();
          create_pipeline ();

          /*if(nvgstcameraParam.vsrc)
             pad = gst_element_get_static_pad (nvgstcameraParam.vsrc, "vidsrc");
             g_object_set(G_OBJECT(nvgstcameraParam.capturefilter), "caps", caps, NULL);

             if(pad){
             gst_pad_send_event (pad, gst_event_new_custom (GST_EVENT_CUSTOM_BOTH,
             gst_structure_new ("renegotiate", NULL)));
             gst_object_unref (pad);
             }
           */
        }
        if (nvgstcameraParam.video_enc == FORMAT_H264_HW) {
          GstCaps *h264enc_caps = NULL;

          //height should be multiple of 16
          nvgstcameraParam.video_capture_height =
              GST_ROUND_UP_16 (nvgstcameraParam.video_capture_height);
          h264enc_caps =
              gst_caps_new_simple ("video/x-h264", "width", G_TYPE_INT,
              nvgstcameraParam.video_capture_width, "height", G_TYPE_INT,
              nvgstcameraParam.video_capture_height, "stream-format",
              G_TYPE_STRING, "byte-stream", "alignment", G_TYPE_STRING, "au",
              NULL);

          g_object_set (G_OBJECT (nvgstcameraParam.h264_filter),
              "caps", h264enc_caps, NULL);
          gst_caps_unref (h264enc_caps);
        }

        gst_caps_unref (caps);
      }
    } else if (!g_strcmp0 (buf, "gicr")) {
      g_print ("w = %d, h = %d\n", nvgstcameraParam.image_capture_width,
          nvgstcameraParam.image_capture_height);

    } else if (!g_strcmp0 (buf, "gvcr")) {
      g_print ("w = %d, h = %d\n", nvgstcameraParam.video_capture_width,
          nvgstcameraParam.video_capture_height);

    } else if (g_str_has_prefix (buf, "mo:")) {
      nvgstcameraParam.mode = atoi (buf + 3);
      if (nvgstcameraParam.mode != 1 && nvgstcameraParam.mode != 2) {
        nvgstcameraParam.mode = NVGST_DEFAULT_CAPTURE_MODE;
        g_print ("Invalid input mode, setting mode to image-capture = 1 \n");
      }

      if (nvgstUsbCamParam.usbcam_enable || nvgstUsbCamParam.use_testsrc
          || nvgstCSIParam.csi_enable) {
        if (nvgstcameraParam.use_camerabin) {
          g_object_set (G_OBJECT (nvgstcameraParam.camera),
              "mode", nvgstcameraParam.mode, NULL);
        } else {
          restart_usbcam_native_pipeline ();
        }
      } else {
        if (nvgstcameraParam.use_camerabin) {
          g_object_set (G_OBJECT (nvgstcameraParam.camera),
              "mode", nvgstcameraParam.mode, NULL);
        } else {
          g_object_set (G_OBJECT (nvgstcameraParam.vsrc),
              "mode", nvgstcameraParam.mode, NULL);
        }
        g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "framerate",
            nvgstcameraParam.fps_rate, NULL);
      }
    } else if (!g_strcmp0 (buf, "gmo")) {
      g_print ("mo = %d\n", nvgstcameraParam.mode);
      g_print (" (2): video\n (1): image\n ");

    } else if (g_str_has_prefix (buf, "cdn:")) {

      if (nvgstUsbCamParam.cap_dev_node) {
        g_free (nvgstUsbCamParam.cap_dev_node);
        nvgstUsbCamParam.cap_dev_node = NULL;
      }

      nvgstUsbCamParam.cap_dev_node = g_strdup (buf + 4);
      set_capture_device_node ();

      g_print ("Set Capture Device = %s\n", nvgstUsbCamParam.vidcap_device);
      restart_usbcam_native_pipeline ();
    } else if (!g_strcmp0 (buf, "gcdn")) {
      g_print ("Capture Device Node = %s\n", nvgstUsbCamParam.vidcap_device);

    } else if (g_str_has_prefix (buf, "fr:")) {
      val = atoi (buf + 3);
      if (val < 5 || val > 30) {
        nvgstcameraParam.fps_rate = NVGST_DEFAULT_FPS;
      } else {
        nvgstcameraParam.fps_rate = val;
      }
      g_print ("frame-rate = %d\n", nvgstcameraParam.fps_rate);
      if (nvgstUsbCamParam.usbcam_enable ||
          nvgstUsbCamParam.use_testsrc || nvgstCSIParam.csi_enable) {
        g_object_set (G_OBJECT (nvgstUsbCamParam.vsrc), "framerate",
            nvgstcameraParam.fps_rate, NULL);
      } else {
        g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "framerate",
            nvgstcameraParam.fps_rate, NULL);
      }

    } else if (!g_strcmp0 (buf, "gfr")) {
      g_print ("frame-rate = %d\n", nvgstcameraParam.fps_rate);

    } else if (g_str_has_prefix (buf, "k:")) {
      destroy_pipeline ();
      nvgstcameraParam.file_type = atoi (buf + 2);
      if (nvgstcameraParam.file_type < 0 || nvgstcameraParam.file_type > 2) {
        nvgstcameraParam.file_type = NVGST_DEFAULT_FILE_TYPE;
        g_print
            ("Invalid input value of Container format, setting container-format"
            " to auto-value =0 (mp4) \n");
      }
      g_print ("file-type = %d\n", nvgstcameraParam.file_type);
      if (!create_pipeline ())
        g_print ("Warning: Not a valid file type\n");

    } else if (!g_strcmp0 (buf, "gk")) {
      g_print ("file-type = %d\n", nvgstcameraParam.file_type);

#ifdef CSI_CAPTURE_SUPPORTED
    } else if (g_str_has_prefix (buf, "px:")) {
      val = atoi (buf + 3);
      if (val != 0 && val != 90 && val != 180 && val != 270) {
        val = NVGST_DEFAULT_PREVIEW_ROTATION;
        g_print
            ("Invalid input value of Rotation preview, setting preview-rotation"
            " to auto-value = %d\n", val);
      }
      g_print ("val = %d\n", val);
      g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "rotate-viewfinder", val,
          NULL);

    } else if (!g_strcmp0 (buf, "gpx")) {
      g_object_get (G_OBJECT (nvgstcameraParam.vsrc), "rotate-viewfinder", &val,
          NULL);
      g_print ("val = %d\n", val);

    } else if (g_str_has_prefix (buf, "cx:")) {
      val = atoi (buf + 3);
      if (val != 0 && val != 90 && val != 180 && val != 270) {
        val = NVGST_DEFAULT_CAPTURE_ROTATION;
        g_print
            ("Invalid input value of Rotation capture, setting capture-rotation"
            " to auto-value = %d\n", val);
      }
      g_print ("val = %d\n", val);
      g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "rotate-capture", val,
          NULL);

    } else if (!g_strcmp0 (buf, "gcx")) {
      g_object_get (G_OBJECT (nvgstcameraParam.vsrc), "rotate-capture", &val,
          NULL);
      g_print ("val = %d\n", val);

    } else if (g_str_has_prefix (buf, "ex:")) {
      val = atoi (buf + 3);
      if (val < 0 || val > 0x7fffffff) {
        val = NVGST_DEFAULT_EXPOSURE;
        g_print ("Invalid input value of exposure, setting exposure"
            " to auto-value = %d\n", val);
      }
      g_print ("val = %d\n", val);
      g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "exposure", val, NULL);

    } else if (!g_strcmp0 (buf, "gex")) {
      g_object_get (G_OBJECT (nvgstcameraParam.vsrc), "exposure", &val, NULL);
      g_print ("val = %d\n", val);

    } else if (g_str_has_prefix (buf, "wb:")) {
      val = atoi (buf + 3);
      if (val < 0 || val > 10) {
        val = NVGST_DEFAULT_WHITEBALANCE;
        g_print ("Invalid input value of white balance, setting white-balance"
            " to auto-value =0 \n");
      }
      g_print ("val = %d\n", val);

      g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "white-balance-mode", val,
          NULL);

    } else if (!g_strcmp0 (buf, "gwb")) {
      g_object_get (G_OBJECT (nvgstcameraParam.vsrc), "white-balance-mode",
          &val, NULL);
      g_print ("val = %d\n", val);

    } else if (g_str_has_prefix (buf, "fc:")) {
      val = atoi (buf + 3);
      g_print ("val = %d\n", val);
      g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "focus", val, NULL);

    } else if (!g_strcmp0 (buf, "gfc")) {
      g_object_get (G_OBJECT (nvgstcameraParam.vsrc), "focus", &val, NULL);
      g_print ("val = %d\n", val);

    } else if (g_str_has_prefix (buf, "io:")) {
      val = atoi (buf + 3);
      if (val != 0 && val != 100 && val != 200 && val != 400 && val != 800) {
        val = NVGST_DEFAULT_ISO;
        g_print ("Invalid input value of ISO mode, setting ISO-mode"
            " to auto-value = 0 \n");
      }
      g_print ("val = %d\n", val);

      g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "iso-speed", val, NULL);

    } else if (!g_strcmp0 (buf, "gio")) {
      g_object_get (G_OBJECT (nvgstcameraParam.vsrc), "iso-speed", &val, NULL);
      g_print ("val = %d\n", val);

    } else if (g_str_has_prefix (buf, "mt:")) {
      val = atoi (buf + 3);
      if (val < 0 || val > 2) {
        val = NVGST_DEFAULT_METERMODE;
        g_print
            ("Invalid input value of metering mode,setting metering to auto-value = 0 \n");
      }
      g_print ("val = %d\n", val);
      g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "metering-mode", val,
          NULL);

    } else if (!g_strcmp0 (buf, "gmt")) {
      g_object_get (G_OBJECT (nvgstcameraParam.vsrc), "metering-mode", &val,
          NULL);
      g_print ("val = %d\n", val);

    } else if (g_str_has_prefix (buf, "ev:")) {
      fval = atof (buf + 3);
      if (fval < -2.5 || fval > 2.5) {
        fval = NVGST_DEFAULT_EVCOMP;
        g_print ("Invalid value of set for ev compensation,setting"
            " to default value =0 \n");
      }
      g_print ("val = %f\n", fval);

      g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "ev-compensation", fval,
          NULL);

    } else if (!g_strcmp0 (buf, "gev")) {
      g_object_get (G_OBJECT (nvgstcameraParam.vsrc), "ev-compensation", &fval,
          NULL);
      g_print ("fval = %f\n", fval);

    } else if (g_str_has_prefix (buf, "bt:")) {
      val = atoi (buf + 3);
      if (val < 0 || val > 100) {
        val = NVGST_DEFAULT_BRIGHTNESS;
        g_print ("Invalid value of set for brightness,setting"
            " to default value =33 \n");
      }
      g_print ("val = %d\n", val);
      g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "brightness", val, NULL);

    } else if (!g_strcmp0 (buf, "gbt")) {
      g_object_get (G_OBJECT (nvgstcameraParam.vsrc), "brightness", &val, NULL);
      g_print ("val = %d\n", val);

    } else if (g_str_has_prefix (buf, "st:")) {
      val = atoi (buf + 3);
      if (val < -100 || val > 100) {
        val = NVGST_DEFAULT_SATURATION;
        g_print ("Invalid value of set for saturation,setting"
            " to default value = 0 \n");
      }
      g_print ("val = %d\n", val);
      g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "saturation", val, NULL);

    } else if (!g_strcmp0 (buf, "gst")) {
      g_object_get (G_OBJECT (nvgstcameraParam.vsrc), "saturation", &val, NULL);
      g_print ("val = %d\n", val);

    } else if (g_str_has_prefix (buf, "ct:")) {
      val = atoi (buf + 3);
      if (val < 0 || val > 100) {
        val = NVGST_DEFAULT_CONTRAST;
        g_print ("Invalid value of set for contrast,setting"
            " to default value = 0 \n");
      }
      g_print ("val = %d\n", val);
      g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "contrast", val, NULL);

    } else if (!g_strcmp0 (buf, "gct")) {
      g_object_get (G_OBJECT (nvgstcameraParam.vsrc), "contrast", &val, NULL);
      g_print ("val = %d\n", val);

    } else if (g_str_has_prefix (buf, "ht:")) {
      val = atoi (buf + 3);
      if (val < 0 || val > 100) {
        val = NVGST_DEFAULT_HUE;
        g_print ("Invalid value of set for hue,setting "
            "to default value =0 \n");
      }
      g_print ("val = %d\n", val);
      g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "hue", val, NULL);

    } else if (!g_strcmp0 (buf, "ght")) {
      g_object_get (G_OBJECT (nvgstcameraParam.vsrc), "hue", &val, NULL);
      g_print ("val = %d\n", val);

    } else if (g_str_has_prefix (buf, "fl:")) {
      val = atoi (buf + 3);
      g_print ("val = %d\n", val);
      if (val < 0 || val > 3) {
        val = NVGST_DEFAULT_FLICKER;
        g_print ("Invalid value of set for flicker control,setting"
            " to default value =3 \n");
      }
      g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "flicker-mode", val,
          NULL);

    } else if (!g_strcmp0 (buf, "gfl")) {
      {
        g_object_get (G_OBJECT (nvgstcameraParam.vsrc), "flicker-mode", &val,
            NULL);
        g_print ("val = %d\n", val);
      }

    } else if (g_str_has_prefix (buf, "ee:")) {
      fval = atof (buf + 3);
      if (fval < -1.0 || fval > 1.0) {
        fval = NVGST_DEFAULT_EDGEENHANCEMENT;
        g_print ("Invalid value of set for edge enhancement,setting"
            " to default value = 0 \n");
      }
      g_print ("val = %f\n", fval);
      g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "edge-enhancement", fval,
          NULL);

    } else if (!g_strcmp0 (buf, "gee")) {
      g_object_get (G_OBJECT (nvgstcameraParam.vsrc), "edge-enhancement", &fval,
          NULL);
      g_print ("val = %f\n", fval);

    } else if (g_str_has_prefix (buf, "xx:")) {
      val = atoi (buf + 3);
      if (val < 0 || val > 5) {
        val = NVGST_DEFAULT_FLASH_MODE;
        g_print
            ("Invalid value of set for flash,setting to default value = 1 \n");
      }
      g_print ("val = %d\n", val);

      g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "flash-mode", val, NULL);

    } else if (!g_strcmp0 (buf, "gxx")) {
      g_object_get (G_OBJECT (nvgstcameraParam.vsrc), "flash-mode", &val, NULL);
      g_print ("val = %d\n", val);

    } else if (g_str_has_prefix (buf, "nr:")) {
      val = atoi (buf + 3);
      if (val < 0 || val > 1) {
        val = NVGST_DEFAULT_NR;
        g_print
            ("Invalid value of set for noise reduction,setting to default value =0 \n");
      }
      g_print ("val = %d\n", val);

      if (val == 1) {
        g_object_get (G_OBJECT (nvgstcameraParam.vsrc), "noise-reduction", &val,
            NULL);
        g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "noise-reduction",
            val | 2, NULL);
      } else
        g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "noise-reduction", 0,
            NULL);

    } else if (!g_strcmp0 (buf, "gnr")) {
      g_object_get (G_OBJECT (nvgstcameraParam.vsrc), "noise-reduction", &val,
          NULL);
      g_print ("val = %d\n", val & 2 ? 1 : 0);

    } else if (g_str_has_prefix (buf, "fm:")) {
      val = atoi (buf + 3);
      if (val < 0 || val > 9) {
        val = NVGST_DEFAULT_FOCUS_MODE;
        g_print
            ("Invalid value of set for focus mode,setting to default value =0 \n");
      }
      g_print ("val = %d\n", val);
      g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "focus-mode", val, NULL);

    } else if (!g_strcmp0 (buf, "gfm")) {
      g_object_get (G_OBJECT (nvgstcameraParam.vsrc), "focus-mode", &val, NULL);
      g_print ("val = %d\n", val);
    } else if (g_str_has_prefix (buf, "scm:")) {
      val = atoi (buf + 4);
      if (val < 0 || val > 16) {
        val = NVGST_DEFAULT_SCENE_MODE;
        g_print
            ("Invalid value of set for Scene mode,setting to default value =0 \n");
      }
      g_print ("val = %d\n", val);
      g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "scene-mode", val, NULL);

    } else if (!g_strcmp0 (buf, "gscm")) {
      g_object_get (G_OBJECT (nvgstcameraParam.vsrc), "scene-mode", &val, NULL);
      g_print ("val = %d\n", val);

    } else if (g_str_has_prefix (buf, "ft:")) {
      val = atoi (buf + 3);
      if (val < 0 || val > 23) {
        val = NVGST_DEFAULT_COLOR_MODE;
        g_print
            ("Invalid value set for Color-tone-mode,setting to default value =0 \n");
      }
      g_print ("val = %d\n", val);
      g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "colour-tone-mode", val,
          NULL);

    } else if (!g_strcmp0 (buf, "gft")) {
      g_object_get (G_OBJECT (nvgstcameraParam.vsrc), "colour-tone-mode", &val,
          NULL);
      g_print ("val = %d\n", val);

    } else if (g_str_has_prefix (buf, "saf:")) {
      val = atoi (buf + 4);
      g_print ("val = %d\n", val);
      if (val)
        g_signal_emit_by_name (G_OBJECT (nvgstcameraParam.vsrc),
            "start-auto-focus", 0);
      else
        g_signal_emit_by_name (G_OBJECT (nvgstcameraParam.vsrc),
            "stop-auto-focus", 0);

    } else if (g_str_has_prefix (buf, "ae:")) {
      destroy_pipeline ();
      nvgstcameraParam.audio_enc = atoi (buf + 3);
      if (nvgstcameraParam.audio_enc < 0 || nvgstcameraParam.audio_enc > 1) {
        g_print ("Warning: Not a valid option for audio encoder,setting"
            " to default value 0(aac)\n");
        nvgstcameraParam.audio_enc = NVGST_DEFAULT_AUDIO_ENCODER;
      }
      if (!create_pipeline ())
        g_print ("Warning: Not a valid audio encoder\n");

    } else if (!g_strcmp0 (buf, "gae")) {
      g_print ("ae: = %d\n", nvgstcameraParam.audio_enc);
#endif
    } else if (g_str_has_prefix (buf, "ve:")) {
      if (!(nvgstUsbCamParam.usbcam_enable || nvgstUsbCamParam.use_testsrc))
        destroy_pipeline ();
      nvgstcameraParam.video_enc = atoi (buf + 3);
      if (nvgstcameraParam.video_enc < FORMAT_H264_HW
          || nvgstcameraParam.video_enc > FORMAT_H263_SW) {
        g_print ("Warning: Not a valid option for video encoder,setting"
            " to default value 0(h264)\n");
        nvgstcameraParam.video_enc = NVGST_DEFAULT_VIDEO_ENCODER;
      }
      if (nvgstUsbCamParam.usbcam_enable || nvgstUsbCamParam.use_testsrc)
        restart_usbcam_native_pipeline ();
      else if (!create_pipeline ())
        g_print ("Waring: Not a valid video encoder\n");

    } else if (!g_strcmp0 (buf, "gve")) {
      g_print ("ve: = %d\n", nvgstcameraParam.video_enc);

    } else if (g_str_has_prefix (buf, "ie:")) {
      destroy_pipeline ();
      nvgstcameraParam.image_enc = atoi (buf + 3);
      if (nvgstcameraParam.image_enc < 0 || nvgstcameraParam.image_enc > 1) {
        g_print ("Warning: Not a valid option for image encoder,setting"
            " to default value 0(jpeg_HW)\n");
        nvgstcameraParam.image_enc = NVGST_DEFAULT_IMAGE_ENCODER;
      }
      if (!create_pipeline ())
        g_print ("Warning: Not a valid image encoder\n");

    } else if (!g_strcmp0 (buf, "gie")) {
      g_print ("ie: = %d\n", nvgstcameraParam.image_enc);
#ifdef CSI_CAPTURE_SUPPORTED
    } else if (g_str_has_prefix (buf, "stm:")) {
      destroy_pipeline ();
      nvgstcameraParam.stereomode = atoi (buf + 4);
      if (nvgstcameraParam.stereomode < 0 || nvgstcameraParam.stereomode > 3) {
        nvgstcameraParam.stereomode = NVGST_DEFAULT_STEREO_MODE;
        g_print ("Invalid value set for Stereo mode,setting"
            " to default value =0 (off) \n");
      }
      if (!create_pipeline ())
        g_print ("Waring: Not a valid stereo mode\n");

    } else if (!g_strcmp0 (buf, "gstm")) {
      g_object_get (G_OBJECT (nvgstcameraParam.vsrc), "stereo-mode", &val,
          NULL);
      g_print ("stereo mode = %d\n", val);

    } else if (!g_strcmp0 (buf, "gsid")) {
      g_print ("sensor id = %d\n", nvgstcameraParam.sensorid);

    } else if (!g_strcmp0 (buf, "gac")) {
#endif
    }
  }

  return TRUE;

invalid_input:
  g_print ("Invalid input\n");
  return TRUE;
}

static void
destroy_pipeline ()
{
  recording = FALSE;
  gst_element_set_state (nvgstcameraParam.camera, GST_STATE_NULL);
  gst_object_unref (GST_OBJECT (nvgstcameraParam.camera));
  nvgstcameraParam.camera = NULL;
  nvgstcameraParam.vsink = NULL;
  nvgstcameraParam.vsrc = NULL;
  nvgstcameraParam.audio_src = NULL;
  nvgstcameraParam.vencoder = NULL;
  nvgstcameraParam.iencoder = NULL;
  nvgstcameraParam.h264_filter = NULL;
  nvgstcameraParam.h264parse = NULL;
  nvgstcameraParam.video_sink = NULL;
  nvgstcameraParam.image_filter = NULL;
  nvgstcameraParam.capturefilter = NULL;
  nvgstcameraParam.audiofilter = NULL;
  nvgstcameraParam.muxer = NULL;
  nvgstcameraParam.aencoder = NULL;
  nvgstcameraParam.colorspace_conv = NULL;
  if (nvgstcameraParam.extra_options)
    g_free (nvgstcameraParam.extra_options);
  nvgstcameraParam.extra_options = NULL;

  if (nvgstcameraParam.qp_range) {
    g_free (nvgstcameraParam.qp_range);
    nvgstcameraParam.qp_range = NULL;
  }
}

static void
destroy_usbcam_pipeline ()
{
  recording = FALSE;
  nvgstUsbCamParam.usbcam_enable = FALSE;
  nvgstUsbCamParam.usbcam_record = FALSE;
  nvgstUsbCamParam.use_testsrc = FALSE;

  if (GST_STATE_CHANGE_FAILURE ==
      gst_element_set_state (nvgstcameraParam.camera, GST_STATE_NULL)) {
    g_warning ("can't set usb cam pipeline to null\n");
  }
  gst_object_unref (GST_OBJECT (nvgstcameraParam.camera));
  nvgstcameraParam.camera = NULL;
  nvgstcameraParam.vencoder = NULL;
  nvgstcameraParam.video_sink = NULL;
  nvgstcameraParam.muxer = NULL;
  nvgstUsbCamParam.usbcam_capturefilter = NULL;
  nvgstUsbCamParam.usbcam_prev_queue = NULL;
  nvgstUsbCamParam.usbcam_cap_queue = NULL;
  nvgstUsbCamParam.usbcam_tee = NULL;
  nvgstUsbCamParam.nv_vidconv = NULL;
  nvgstUsbCamParam.encoder = NULL;
  nvgstUsbCamParam.colorspace_conv = NULL;
  nvgstUsbCamParam.vsrc = NULL;
  nvgstUsbCamParam.vsink = NULL;
  if (nvgstUsbCamParam.vidcap_device) {
    g_free (nvgstUsbCamParam.vidcap_device);
    nvgstUsbCamParam.vidcap_device = NULL;
  }
  if (nvgstUsbCamParam.cap_dev_node) {
    g_free (nvgstUsbCamParam.cap_dev_node);
    nvgstUsbCamParam.cap_dev_node = NULL;
  }
  if (nvgstcameraParam.extra_options)
    g_free (nvgstcameraParam.extra_options);
  nvgstcameraParam.extra_options = NULL;
  if (nvgstcameraParam.qp_range) {
    g_free (nvgstcameraParam.qp_range);
    nvgstcameraParam.qp_range = NULL;
  }
}

static void
destroy_CSIcam_pipeline ()
{
  destroy_usbcam_pipeline ();
}

static GstBusSyncReply
bus_sync_handler (GstBus * bus, GstMessage * msg, gpointer data)
{
  switch (GST_MESSAGE_TYPE (msg)) {
    case GST_MESSAGE_ELEMENT:
      if (GST_MESSAGE_SRC (msg) == GST_OBJECT (nvgstcameraParam.camera)) {
        const GstStructure *structure;
        structure = gst_message_get_structure (msg);
        if (gst_structure_has_name (structure, "image-done")) {
          g_print ("%s : got image-done\n", __func__);
          g_mutex_lock (nvgstcameraParam.lock);

          recording = FALSE;
          g_cond_signal (nvgstcameraParam.cond);
          g_mutex_unlock (nvgstcameraParam.lock);

        } else if (gst_structure_has_name (structure, "video-done")) {
          g_print ("got video-done\n");
        }
      }
      return GST_BUS_PASS;

    default:
      return GST_BUS_PASS;
  }
}

static void
image_captured (GstElement * fsink, GstBuffer * buffer,
    GstPad * pad, gpointer udata)
{
  g_print ("============> image captured <========= \n");

  if (GST_BUFFER_SIZE (buffer)) {
    FILE *fp;
    gchar outfile[100];
    gchar temp[100];
    memset (outfile, 0, sizeof (outfile));
    memset (temp, 0, sizeof (temp));

    strcat (outfile, nvgstcameraParam.file_name);
    sprintf (temp, "%05d.jpg", nvgstcameraParam.capture_count++);
    strcat (outfile, temp);
    fp = fopen (outfile, "wb");
    fwrite (GST_BUFFER_DATA (buffer), 1, GST_BUFFER_SIZE (buffer), fp);
    fclose (fp);
  }

  g_mutex_lock (nvgstcameraParam.lock);
  recording = FALSE;
  g_cond_signal (nvgstcameraParam.cond);
  g_mutex_unlock (nvgstcameraParam.lock);

}

static void
USBCam_image_captured (GstElement * fsink, GstBuffer * buffer,
    GstPad * pad, gpointer udata)
{
  if (nvgstUsbCamParam.usbcam_capcount == 0) {
    g_print ("============> USBCam image captured <========= \n");

    if (GST_BUFFER_SIZE (buffer)) {
      FILE *fp;
      gchar outfile[100];
      gchar temp[100];
      memset (outfile, 0, sizeof (outfile));
      memset (temp, 0, sizeof (temp));

      strcat (outfile, nvgstcameraParam.file_name);
      sprintf (temp, "%05d.jpg", nvgstcameraParam.capture_count++);
      strcat (outfile, temp);

      fp = fopen (outfile, "wb");
      fwrite (GST_BUFFER_DATA (buffer), 1, GST_BUFFER_SIZE (buffer), fp);
      fclose (fp);
    }

    nvgstUsbCamParam.usbcam_capcount++;
    nvgstUsbCamParam.usbcam_record = FALSE;

    g_mutex_lock (nvgstcameraParam.lock);
    recording = FALSE;
    g_cond_signal (nvgstcameraParam.cond);
    g_mutex_unlock (nvgstcameraParam.lock);
  }
}

#ifdef USE_CAMERABIN2
static GstEncodingProfile *
create_image_profile (gint type)
{
  GstEncodingContainerProfile *prof;
  GstEncodingVideoProfile *vprof;
  GstCaps *caps;

  switch (type) {
    case 0:
      caps = gst_caps_new_simple ("image/jpeg", NULL);
      prof = gst_encoding_container_profile_new ("jpeg", NULL, caps, NULL);
      vprof = gst_encoding_video_profile_new (caps, NULL, NULL, 0);
      break;
    case 1:
      caps = gst_caps_new_simple ("image/png", NULL);
      prof = gst_encoding_container_profile_new ("png", NULL, caps, NULL);
      vprof = gst_encoding_video_profile_new (caps, NULL, NULL, 0);
      break;
    default:
      caps = gst_caps_new_simple ("image/jpeg", NULL);
      prof = gst_encoding_container_profile_new ("jpeg", NULL, caps, NULL);
      vprof = gst_encoding_video_profile_new (caps, NULL, NULL, 0);
      break;
  }
  gst_encoding_video_profile_set_variableframerate (vprof, TRUE);
  gst_encoding_container_profile_add_profile (prof,
      (GstEncodingProfile *) vprof);

  gst_caps_unref (caps);
  return (GstEncodingProfile *) prof;
}



static GstEncodingProfile *
create_video_profile (gint muxer_type, gint video_encoder, gint audio_encoder)
{
  GstEncodingContainerProfile *prof;
  GstEncodingVideoProfile *vprof;
  GstEncodingAudioProfile *aprof;
  GstCaps *caps;

  if ((muxer_type != 1) && (audio_encoder == 1)) {
    g_print ("\nWarning: AMR is not supported with .mp4 and .avi using AAC\n");
    audio_encoder = 0;
  }
  switch (muxer_type) {
    case 0:
      caps =
          gst_caps_new_simple ("video/quicktime", "variant", G_TYPE_STRING,
          "iso", NULL);
      prof = gst_encoding_container_profile_new ("mp4", NULL, caps, NULL);
      break;
    case 1:
      caps =
          gst_caps_new_simple ("video/quicktime", "variant", G_TYPE_STRING,
          "3gpp", NULL);
      prof = gst_encoding_container_profile_new ("3gpp", NULL, caps, NULL);
      break;
    case 2:
      caps = gst_caps_from_string ("video/x-msvideo");
      prof = gst_encoding_container_profile_new ("avi", NULL, caps, NULL);
      break;
    default:
      caps =
          gst_caps_new_simple ("video/quicktime", "variant", G_TYPE_STRING,
          "iso", NULL);
      prof = gst_encoding_container_profile_new ("mp4", NULL, caps, NULL);
      break;
  }
  gst_caps_unref (caps);

  switch (video_encoder) {
    case FORMAT_H264_HW:
      caps = gst_caps_new_simple ("video/x-h264", NULL);
      vprof = gst_encoding_video_profile_new (caps, NULL, NULL, 0);
      gst_encoding_video_profile_set_variableframerate (vprof, TRUE);
      break;
    case FORMAT_VP8_HW:
      caps = gst_caps_new_simple ("video/x-vp8", NULL);
      vprof = gst_encoding_video_profile_new (caps, NULL, NULL, 0);
      gst_encoding_video_profile_set_variableframerate (vprof, TRUE);
      break;
    default:
      caps = gst_caps_new_simple ("video/x-h264", NULL);
      vprof = gst_encoding_video_profile_new (caps, NULL, NULL, 0);
      gst_encoding_video_profile_set_variableframerate (vprof, TRUE);
      break;
  }

  if (!gst_encoding_container_profile_add_profile (prof,
          (GstEncodingProfile *) vprof))
    GST_WARNING_OBJECT (nvgstcameraParam.camera,
        "Failed to create encoding profiles");

  gst_caps_unref (caps);
  if (nvgstcameraParam.Voice_Control) {
    switch (audio_encoder) {
      case 0:
        caps =
            gst_caps_new_simple ("audio/mpeg", "mpegversion", G_TYPE_INT, 4,
            NULL);
        aprof = gst_encoding_audio_profile_new (caps, NULL, NULL, 1);
        break;
      case 1:
        caps = gst_caps_new_simple ("audio/AMR", NULL);
        aprof = gst_encoding_audio_profile_new (caps, NULL, NULL, 0);
        break;

      default:
        caps =
            gst_caps_new_simple ("audio/mpeg", "mpegversion", G_TYPE_INT, 4,
            NULL);
        aprof = gst_encoding_audio_profile_new (caps, NULL, NULL, 1);
        break;
    }

    if (!gst_encoding_container_profile_add_profile (prof,
            (GstEncodingProfile *) aprof)) {
      GST_WARNING_OBJECT (nvgstcameraParam.camera,
          "Failed to create encoding profiles");
    }

    gst_caps_unref (caps);
  }

  return (GstEncodingProfile *) prof;
}
#endif


void
set_new_image_file_name (int type)
{
  gchar filename[100];
  gchar temp[100];
  memset (filename, 0, sizeof (filename));
  memset (temp, 0, sizeof (temp));
  strcat (filename, nvgstcameraParam.file_name);

  switch (type) {
    case 0:
      sprintf (temp, "%05d.jpg", nvgstcameraParam.capture_count++);
      break;
    case 1:
      sprintf (temp, "%05d.png", nvgstcameraParam.capture_count++);
      break;
    default:
      sprintf (temp, "%05d.jpg", nvgstcameraParam.capture_count++);
      break;
  }
  strcat (filename, temp);
  g_object_set (G_OBJECT (nvgstcameraParam.camera), "location", filename, NULL);
}


void
set_new_video_file_name (int type)
{
  gchar filename[100];
  gchar temp[100];
  memset (filename, 0, sizeof (filename));
  memset (temp, 0, sizeof (temp));
  strcat (filename, nvgstcameraParam.file_name);

  switch (type) {
    case 0:
      sprintf (temp, "%05d.mp4", nvgstcameraParam.capture_count++);
      break;
    case 1:
      sprintf (temp, "%05d.3gp", nvgstcameraParam.capture_count++);
      break;
    case 2:
      sprintf (temp, "%05d.avi", nvgstcameraParam.capture_count++);
      break;
    default:
      sprintf (temp, "%05d.mp4", nvgstcameraParam.capture_count++);
      break;
  }
  strcat (filename, temp);
  if (nvgstcameraParam.use_camerabin)
    g_object_set (G_OBJECT (nvgstcameraParam.camera), "location", filename,
        NULL);
  else {
    gst_element_set_state (nvgstcameraParam.video_sink, GST_STATE_NULL);
    g_object_set (G_OBJECT (nvgstcameraParam.video_sink), "location", filename,
        NULL);
    gst_element_set_locked_state (nvgstcameraParam.video_sink, FALSE);
    gst_element_set_state (nvgstcameraParam.video_sink, GST_STATE_PLAYING);
  }
}


static gboolean
get_image_encoder (GstElement ** iencoder)
{
  switch (nvgstcameraParam.image_enc) {
    case 0:
      *iencoder = gst_element_factory_make ("nv_omx_jpegenc", NULL);
      break;
    case 1:
      *iencoder = gst_element_factory_make ("jpegenc", NULL);
      break;
    case 2:
      *iencoder = gst_element_factory_make ("nvjpegenc", NULL);
      break;
    default:
      *iencoder = gst_element_factory_make ("nvjpegenc", NULL);
      break;
  }
  if (!(*iencoder)) {
    nvgstcameraParam.return_value = -1;
    g_print ("Error: Can't Create image encoder element\n");
    return FALSE;
  }


  return TRUE;
}

static gboolean
get_muxer (GstElement ** muxer)
{
  switch (nvgstcameraParam.file_type) {
    case 0:
      *muxer = gst_element_factory_make ("qtmux", NULL);
      break;
    case 1:
      *muxer = gst_element_factory_make ("qtmux", NULL);
      break;
    case 2:
      *muxer = gst_element_factory_make ("avimux", NULL);
      break;
    default:
      *muxer = gst_element_factory_make ("qtmux", NULL);
      break;
  }
  if (!(*muxer)) {
    nvgstcameraParam.return_value = -1;
    g_print ("Error: Can't Create muxer element\n");
    return FALSE;
  }
  return TRUE;
}

static gboolean
get_video_encoder (GstElement ** vencoder)
{
  switch (nvgstcameraParam.video_enc) {
    case FORMAT_H264_HW:
      *vencoder = gst_element_factory_make ("nv_omx_h264enc", NULL);
      break;
    case FORMAT_VP8_HW:
      *vencoder = gst_element_factory_make ("nv_omx_vp8enc", NULL);
      break;
    case FORMAT_MPEG4_SW:
      *vencoder = gst_element_factory_make ("ffenc_mpeg4", NULL);
      break;
    case FORMAT_H263_SW:
      *vencoder = gst_element_factory_make ("ffenc_h263", NULL);
      break;
    default:
      *vencoder = gst_element_factory_make ("nv_omx_h264enc", NULL);
      break;
  }

  if (!(*vencoder)) {
    nvgstcameraParam.return_value = -1;
    g_print ("Error: Can't Create video encoder element\n");
    return FALSE;
  }

  if (nvgstcameraParam.qp_range)
    g_object_set (G_OBJECT (*vencoder), "qp-range", nvgstcameraParam.qp_range,
        NULL);

  return TRUE;
}

static gboolean
get_audio_encoder (GstElement ** aencoder)
{
  if ((nvgstcameraParam.file_type != 1) && (nvgstcameraParam.audio_enc == 1)) {
    g_print ("\nWarning: AMR is not supported with .mp4 and .avi using AAC\n");
    nvgstcameraParam.audio_enc = 0;
  }
  switch (nvgstcameraParam.audio_enc) {
    case 0:
      *aencoder = gst_element_factory_make ("ffenc_aac", NULL);
      break;
    case 1:
      *aencoder = gst_element_factory_make ("amrnbenc", NULL);
      break;

    default:
      *aencoder = gst_element_factory_make ("ffenc_aac", NULL);
      break;
  }

  if (!(*aencoder)) {
    nvgstcameraParam.return_value = -1;
    g_print ("Error: Can't Create audio encoder element\n");
    return FALSE;
  }

  return TRUE;

}

static void
restart_usbcam_native_pipeline ()
{
  GstPad *sinkpad = NULL;

  if (GST_STATE_CHANGE_FAILURE ==
      gst_element_set_state (nvgstcameraParam.camera, GST_STATE_NULL)) {
    g_warning ("can't set usb cam pipeline to null\n");
  }

  /* Remove buffer probe from capture queue sink pad */
  sinkpad =
      gst_element_get_static_pad (nvgstUsbCamParam.usbcam_cap_queue, "sink");
  gst_pad_remove_buffer_probe (sinkpad, nvgstUsbCamParam.handler_id);
  gst_object_unref (sinkpad);
  gst_object_unref (GST_OBJECT (nvgstcameraParam.camera));
  nvgstcameraParam.camera = NULL;

  g_usleep (250000);

  if (nvgstCSIParam.csi_enable) {
    if (!create_CSIcam_pipeline ()) {
      nvgstcameraParam.return_value = -1;
      g_main_loop_quit (loop);
    }
  } else if (nvgstUsbCamParam.usbcam_enable || nvgstUsbCamParam.use_testsrc) {
    if (!create_usbcam_pipeline ()) {
      nvgstcameraParam.return_value = -1;
      g_main_loop_quit (loop);
    }
  }
}

static gboolean
encoder_buf_prob (GstPad * pad, GstBuffer * buffer, gpointer u_data)
{
  return nvgstUsbCamParam.usbcam_record;
}


static gboolean
create_mjpeg_USBcam_pipeline (void)
{
  GstPad *srcpad = NULL;
  GstPad *sinkpad = NULL;
  GstPad *tee_prev_pad = NULL;
  GstPad *tee_vid_pad = NULL;
  GstBus *bus;
  GstCaps *caps;
  gint capture_width;
  gint capture_height;
  GstElement *image_sink = NULL;
  GstElement *jpeg_decoder = NULL;
  GstPadTemplate *tee_src_pad_template = NULL;

  /* Create the empty pipeline */
  nvgstcameraParam.camera = gst_pipeline_new ("mjpeg_usbcam_capture");
  if (!nvgstcameraParam.camera)
    goto done;

  bus = gst_pipeline_get_bus (GST_PIPELINE (nvgstcameraParam.camera));
  gst_bus_set_sync_handler (bus, NULL, NULL);
  gst_bus_set_sync_handler (bus, bus_sync_handler, nvgstcameraParam.camera);
  gst_bus_add_watch (bus, bus_call, NULL);
  gst_object_unref (bus);

  /* Create the video capture source element */
  nvgstUsbCamParam.vsrc = gst_element_factory_make ("v4l2src", NULL);
  g_object_set (G_OBJECT (nvgstUsbCamParam.vsrc), "device",
      nvgstUsbCamParam.vidcap_device, NULL);

  /* Create preview chain elements */
  nvgstUsbCamParam.vsink =
      gst_element_factory_make (nvgstcameraParam.svs, NULL);
  nvgstUsbCamParam.usbcam_tee = gst_element_factory_make ("tee", NULL);
  nvgstUsbCamParam.usbcam_prev_queue = gst_element_factory_make ("queue", NULL);
  nvgstUsbCamParam.usbcam_capturefilter =
      gst_element_factory_make ("capsfilter", NULL);
  jpeg_decoder = gst_element_factory_make ("jpegdec", NULL);
  if (!g_strcmp0 ("ximagesink", nvgstcameraParam.svs)) {
    nvgstUsbCamParam.colorspace_conv =
        gst_element_factory_make ("ffmpegcolorspace", NULL);
  } else if (!g_strcmp0 ("xvimagesink", nvgstcameraParam.svs)) {
    nvgstUsbCamParam.colorspace_conv =
        gst_element_factory_make ("ffmpegcolorspace", NULL);
  } else {
    g_print ("video sink not supported\n");
    goto done;
  }

  if (!nvgstUsbCamParam.vsrc || !nvgstUsbCamParam.vsink
      || !nvgstUsbCamParam.usbcam_capturefilter || !nvgstUsbCamParam.usbcam_tee
      || !nvgstUsbCamParam.usbcam_prev_queue
      || !nvgstUsbCamParam.colorspace_conv || !jpeg_decoder) {
    g_print ("Not all Preview chain elements could be created.\n");
    goto done;
  }

  switch (nvgstcameraParam.mode) {
    case CAPTURE_IMAGE:
      capture_width = nvgstcameraParam.image_capture_width;
      capture_height = nvgstcameraParam.image_capture_height;
      break;
    case CAPTURE_VIDEO:
      capture_width = nvgstcameraParam.video_capture_width;
      capture_height = nvgstcameraParam.video_capture_height;
      break;
    default:
      capture_width = nvgstUsbCamParam.preview_width;
      capture_height = nvgstUsbCamParam.preview_height;
      break;
  }

  caps = gst_caps_new_simple ("image/jpeg",
      "width", G_TYPE_INT, capture_width,
      "height", G_TYPE_INT, capture_height, NULL);
  g_object_set (G_OBJECT (nvgstUsbCamParam.usbcam_capturefilter), "caps", caps,
      NULL);
  gst_caps_unref (caps);

  g_object_set (G_OBJECT (nvgstUsbCamParam.vsink), "async", FALSE, NULL);
  g_object_set (G_OBJECT (nvgstUsbCamParam.vsink), "sync", FALSE, NULL);

  g_object_set (G_OBJECT (nvgstUsbCamParam.usbcam_tee), "name", "usbcam_t",
      NULL);
  g_object_set (G_OBJECT (nvgstUsbCamParam.usbcam_prev_queue), "leaky", 2,
      NULL);
  g_object_set (G_OBJECT (nvgstUsbCamParam.usbcam_prev_queue), "max-size-time",
      (guint64) 0, NULL);
  g_object_set (G_OBJECT (nvgstUsbCamParam.usbcam_prev_queue), "max-size-bytes",
      0, NULL);
  g_object_set (G_OBJECT (nvgstUsbCamParam.usbcam_prev_queue),
      "max-size-buffers", 1, NULL);

  gst_bin_add_many (GST_BIN (nvgstcameraParam.camera), nvgstUsbCamParam.vsrc,
      nvgstUsbCamParam.vsink, nvgstUsbCamParam.usbcam_tee,
      nvgstUsbCamParam.usbcam_prev_queue, nvgstUsbCamParam.colorspace_conv,
      nvgstUsbCamParam.usbcam_capturefilter, jpeg_decoder, NULL);

  /* Link Preview chain elements */
  if (gst_element_link (nvgstUsbCamParam.vsrc,
          nvgstUsbCamParam.usbcam_capturefilter) != TRUE
      || gst_element_link (nvgstUsbCamParam.usbcam_capturefilter,
          nvgstUsbCamParam.usbcam_tee) != TRUE
      || gst_element_link (nvgstUsbCamParam.usbcam_prev_queue,
          jpeg_decoder) != TRUE
      || gst_element_link (jpeg_decoder,
          nvgstUsbCamParam.colorspace_conv) != TRUE
      || gst_element_link (nvgstUsbCamParam.colorspace_conv,
          nvgstUsbCamParam.vsink) != TRUE) {
    g_print ("Preview chain elements could not be linked \n");
    goto done;
  }

  tee_src_pad_template =
      gst_element_class_get_pad_template (GST_ELEMENT_GET_CLASS
      (nvgstUsbCamParam.usbcam_tee), "src%d");

  /* Manually link the Tee with preview queue */
  tee_prev_pad =
      gst_element_request_pad (nvgstUsbCamParam.usbcam_tee,
      tee_src_pad_template, NULL, NULL);
  sinkpad =
      gst_element_get_static_pad (nvgstUsbCamParam.usbcam_prev_queue, "sink");
  if (!sinkpad || !tee_prev_pad) {
    goto done;
  }
  if (GST_PAD_LINK_OK != gst_pad_link (tee_prev_pad, sinkpad)) {
    goto done;
  }
  gst_object_unref (sinkpad);
  gst_object_unref (tee_prev_pad);

  nvgstUsbCamParam.usbcam_cap_queue = gst_element_factory_make ("queue", NULL);
  if (!nvgstUsbCamParam.usbcam_cap_queue) {
    g_print ("Capture queue\n");
    goto done;
  }

  gst_bin_add_many (GST_BIN (nvgstcameraParam.camera),
      nvgstUsbCamParam.usbcam_cap_queue, NULL);

  /* Manually link the Tee with capture queue */
  tee_vid_pad =
      gst_element_request_pad (nvgstUsbCamParam.usbcam_tee,
      tee_src_pad_template, NULL, NULL);
  sinkpad =
      gst_element_get_static_pad (nvgstUsbCamParam.usbcam_cap_queue, "sink");
  if (!sinkpad || !tee_vid_pad) {
    goto done;
  }
  if (GST_PAD_LINK_OK != gst_pad_link (tee_vid_pad, sinkpad)) {
    goto done;
  }
  gst_object_unref (sinkpad);
  gst_object_unref (tee_vid_pad);

  sinkpad =
      gst_element_get_static_pad (nvgstUsbCamParam.usbcam_cap_queue, "sink");
  nvgstUsbCamParam.handler_id =
      gst_pad_add_buffer_probe (sinkpad, G_CALLBACK (encoder_buf_prob), NULL);
  gst_object_unref (sinkpad);

  if (nvgstcameraParam.mode == CAPTURE_IMAGE) {
    /* Create image encode chain elements */
    image_sink = gst_element_factory_make ("fakesink", NULL);
    if (!image_sink)
      goto done;
    g_object_set (G_OBJECT (image_sink), "signal-handoffs", TRUE, NULL);
    g_signal_connect (G_OBJECT (image_sink), "handoff",
        G_CALLBACK (USBCam_image_captured), NULL);

    gst_bin_add_many (GST_BIN (nvgstcameraParam.camera), image_sink, NULL);

    if ((gst_element_link (nvgstUsbCamParam.usbcam_cap_queue,
                image_sink)) != TRUE) {
      g_print ("Elements could not link usbcam_cap_queue & image_sink\n");
      goto done;
    }
  } else if (nvgstcameraParam.mode == CAPTURE_VIDEO) {
    /* Create video encode chain elements */
    nvgstcameraParam.video_sink = gst_element_factory_make ("filesink", NULL);

    if (!get_muxer (&nvgstcameraParam.muxer)) {
      g_print ("Video muxer element could not be created.\n");
      goto done;
    }

    if (!nvgstcameraParam.video_sink) {
      g_print ("Video sink elements could not be created.\n");
      goto done;
    }

    g_object_set (G_OBJECT (nvgstcameraParam.video_sink), "location",
        DEFAULT_LOCATION, NULL);
    g_object_set (G_OBJECT (nvgstcameraParam.video_sink), "async", FALSE, NULL);
    g_object_set (G_OBJECT (nvgstcameraParam.video_sink), "sync", FALSE, NULL);

    gst_bin_add_many (GST_BIN (nvgstcameraParam.camera),
        nvgstcameraParam.muxer, nvgstcameraParam.video_sink, NULL);

    if (gst_element_link (nvgstcameraParam.muxer,
            nvgstcameraParam.video_sink) != TRUE) {
      g_print ("Muxer & Video sink elements could not be linked\n");
      goto done;
    }

    srcpad =
        gst_element_get_static_pad (nvgstUsbCamParam.usbcam_cap_queue, "src");
    sinkpad = gst_element_get_request_pad (nvgstcameraParam.muxer, "video_%d");
    if (!sinkpad || !srcpad) {
      goto done;
    }
    if (GST_PAD_LINK_OK != gst_pad_link (srcpad, sinkpad)) {
      goto done;
    }
    gst_object_unref (sinkpad);
    gst_object_unref (srcpad);
  }

  if (GST_STATE_CHANGE_FAILURE ==
      gst_element_set_state (nvgstcameraParam.camera, GST_STATE_PLAYING)) {
    g_warning ("can't set usb cam pipeline to playing\n");
    goto done;
  }
  return TRUE;

done:
  nvgstcameraParam.return_value = -1;
  g_print ("********* DETECTED A PROBLEM WITH MJPEG USB CAM NATIVE PIPELINE\n");
  return FALSE;
}

static gboolean
create_yuv_USBcam_pipeline (void)
{
  GstPad *srcpad = NULL;
  GstPad *sinkpad = NULL;
  GstPad *tee_prev_pad = NULL;
  GstPad *tee_vid_pad = NULL;
  GstCaps *caps;
  GstBus *bus;
  gint capture_width;
  gint capture_height;
  guint32 fourcc = 0;
  GstElement *image_sink = NULL;
  GstElement *cap_convert = NULL;
  GstElement *cap_conv_filter = NULL;
  GstPadTemplate *tee_src_pad_template = NULL;

  nvgstcameraParam.mimetype = "video/x-raw-yuv";
  nvgstcameraParam.preview_mimetype = "video/x-raw-yuv";
  nvgstcameraParam.capture_mimetype = "video/x-nvrm-yuv";

  /* Create the empty pipeline */
  nvgstcameraParam.camera = gst_pipeline_new ("yuv_usbcam_capture");
  if (!nvgstcameraParam.camera)
    goto done;

  bus = gst_pipeline_get_bus (GST_PIPELINE (nvgstcameraParam.camera));
  gst_bus_set_sync_handler (bus, NULL, NULL);
  gst_bus_set_sync_handler (bus, bus_sync_handler, nvgstcameraParam.camera);
  gst_bus_add_watch (bus, bus_call, NULL);
  gst_object_unref (bus);

  /* Capture Format */
  switch (nvgstUsbCamParam.capture_format) {
    case 0:
      fourcc = GST_MAKE_FOURCC ('I', '4', '2', '0');
      break;
    case 1:
      fourcc = GST_MAKE_FOURCC ('Y', 'V', '1', '2');
      break;
    case 2:
      fourcc = GST_MAKE_FOURCC ('N', 'V', '1', '2');
      break;
    case 3:
      fourcc = GST_MAKE_FOURCC ('Y', 'U', 'Y', '2');
      break;
    case 4:
      fourcc = GST_MAKE_FOURCC ('U', 'Y', 'V', 'Y');
      break;
    case 5:
      fourcc = GST_MAKE_FOURCC ('Y', 'V', 'Y', 'U');
      break;
    case 6:
      fourcc = GST_MAKE_FOURCC ('Y', '4', '4', '4');
      break;
    default:
      fourcc = GST_MAKE_FOURCC ('Y', 'U', 'Y', '2');
      break;
  }

  /* Create the video capture source element */
  if (nvgstUsbCamParam.use_testsrc) {
    nvgstUsbCamParam.vsrc = gst_element_factory_make ("videotestsrc", NULL);
    g_object_set (G_OBJECT (nvgstUsbCamParam.vsrc), "is-live", TRUE, NULL);
  } else {
    nvgstUsbCamParam.vsrc = gst_element_factory_make ("v4l2src", NULL);
    g_object_set (G_OBJECT (nvgstUsbCamParam.vsrc), "device",
        nvgstUsbCamParam.vidcap_device, NULL);
    g_object_set (G_OBJECT (nvgstUsbCamParam.vsrc), "queue-size", 4, NULL);     //default is 2
    g_object_set (G_OBJECT (nvgstUsbCamParam.vsrc), "always-copy", FALSE, NULL);
  }

  /* Create preview chain elements */
  nvgstUsbCamParam.vsink =
      gst_element_factory_make (nvgstcameraParam.svs, NULL);
  nvgstUsbCamParam.usbcam_tee = gst_element_factory_make ("tee", NULL);
  nvgstUsbCamParam.usbcam_prev_queue = gst_element_factory_make ("queue", NULL);
  nvgstUsbCamParam.usbcam_capturefilter =
      gst_element_factory_make ("capsfilter", NULL);
  cap_convert = gst_element_factory_make ("ffmpegcolorspace", NULL);
  cap_conv_filter = gst_element_factory_make ("capsfilter", NULL);
  if (!g_strcmp0 ("ximagesink", nvgstcameraParam.svs)) {
    nvgstUsbCamParam.colorspace_conv =
        gst_element_factory_make ("ffmpegcolorspace", NULL);
  } else if (!g_strcmp0 ("xvimagesink", nvgstcameraParam.svs)) {
    nvgstUsbCamParam.colorspace_conv =
        gst_element_factory_make ("ffmpegcolorspace", NULL);
  } else if (!g_strcmp0 ("nv_omx_videosink", nvgstcameraParam.svs)) {
    nvgstUsbCamParam.colorspace_conv =
        gst_element_factory_make ("identity", NULL);
  } else if (!g_strcmp0 ("nv_omx_hdmi_videosink", nvgstcameraParam.svs)) {
    nvgstUsbCamParam.colorspace_conv =
        gst_element_factory_make ("identity", NULL);
  } else {
    g_print ("video sink not supported\n");
    goto done;
  }

  if (!nvgstUsbCamParam.vsrc || !nvgstUsbCamParam.vsink
      || !nvgstUsbCamParam.usbcam_capturefilter || !nvgstUsbCamParam.usbcam_tee
      || !nvgstUsbCamParam.usbcam_prev_queue
      || !nvgstUsbCamParam.colorspace_conv || !cap_convert
      || !cap_conv_filter) {
    g_print ("Not all Preview chain elements could be created.\n");
    goto done;
  }

  switch (nvgstcameraParam.mode) {
    case CAPTURE_IMAGE:
      capture_width = nvgstcameraParam.image_capture_width;
      capture_height = nvgstcameraParam.image_capture_height;
      break;
    case CAPTURE_VIDEO:
      capture_width = nvgstcameraParam.video_capture_width;
      capture_height = nvgstcameraParam.video_capture_height;
      break;
    default:
      capture_width = nvgstUsbCamParam.preview_width;
      capture_height = nvgstUsbCamParam.preview_height;
      break;
  }

  caps = gst_caps_new_simple (nvgstcameraParam.mimetype,
      "format", GST_TYPE_FOURCC, fourcc,
      "width", G_TYPE_INT, capture_width,
      "height", G_TYPE_INT, capture_height, NULL);
  g_object_set (G_OBJECT (nvgstUsbCamParam.usbcam_capturefilter), "caps", caps,
      NULL);
  gst_caps_unref (caps);

  caps = gst_caps_new_simple (nvgstcameraParam.mimetype,
      "format", GST_TYPE_FOURCC, GST_MAKE_FOURCC ('I', '4', '2', '0'),
      "width", G_TYPE_INT, capture_width,
      "height", G_TYPE_INT, capture_height, NULL);
  g_object_set (G_OBJECT (cap_conv_filter), "caps", caps, NULL);
  gst_caps_unref (caps);

  g_object_set (G_OBJECT (nvgstUsbCamParam.vsink), "async", FALSE, NULL);
  g_object_set (G_OBJECT (nvgstUsbCamParam.vsink), "sync", FALSE, NULL);

  g_object_set (G_OBJECT (nvgstUsbCamParam.usbcam_prev_queue), "max-size-time",
      (guint64) 0, NULL);
  g_object_set (G_OBJECT (nvgstUsbCamParam.usbcam_prev_queue), "max-size-bytes",
      0, NULL);
  g_object_set (G_OBJECT (nvgstUsbCamParam.usbcam_prev_queue),
      "max-size-buffers", 1, NULL);

  g_object_set (G_OBJECT (nvgstUsbCamParam.usbcam_tee), "name", "usbcam_t",
      NULL);
  g_object_set (G_OBJECT (nvgstUsbCamParam.usbcam_prev_queue), "leaky", 2,
      NULL);

  gst_bin_add_many (GST_BIN (nvgstcameraParam.camera), nvgstUsbCamParam.vsrc,
      nvgstUsbCamParam.vsink, nvgstUsbCamParam.usbcam_tee,
      nvgstUsbCamParam.usbcam_prev_queue, nvgstUsbCamParam.colorspace_conv,
      nvgstUsbCamParam.usbcam_capturefilter, cap_convert, cap_conv_filter,
      NULL);

  /* Link Preview chain elements */
  if (gst_element_link (nvgstUsbCamParam.vsrc,
          nvgstUsbCamParam.usbcam_capturefilter) != TRUE
      || gst_element_link (nvgstUsbCamParam.usbcam_capturefilter,
          cap_convert) != TRUE
      || gst_element_link (cap_convert, cap_conv_filter) != TRUE
      || gst_element_link (cap_conv_filter, nvgstUsbCamParam.usbcam_tee) != TRUE
      || gst_element_link (nvgstUsbCamParam.usbcam_prev_queue,
          nvgstUsbCamParam.colorspace_conv) != TRUE
      || gst_element_link (nvgstUsbCamParam.colorspace_conv,
          nvgstUsbCamParam.vsink) != TRUE) {
    g_print ("Preview chain elements could not be linked \n");
    goto done;
  }

  tee_src_pad_template =
      gst_element_class_get_pad_template (GST_ELEMENT_GET_CLASS
      (nvgstUsbCamParam.usbcam_tee), "src%d");

  /* Manually link the Tee with preview queue */
  tee_prev_pad =
      gst_element_request_pad (nvgstUsbCamParam.usbcam_tee,
      tee_src_pad_template, NULL, NULL);
  sinkpad =
      gst_element_get_static_pad (nvgstUsbCamParam.usbcam_prev_queue, "sink");
  if (!sinkpad || !tee_prev_pad) {
    goto done;
  }
  if (GST_PAD_LINK_OK != gst_pad_link (tee_prev_pad, sinkpad)) {
    goto done;
  }
  gst_object_unref (sinkpad);
  gst_object_unref (tee_prev_pad);

  nvgstcameraParam.capturefilter =
      gst_element_factory_make ("capsfilter", NULL);
  nvgstUsbCamParam.usbcam_cap_queue = gst_element_factory_make ("queue", NULL);
  if (!nvgstcameraParam.capturefilter || !nvgstUsbCamParam.usbcam_cap_queue) {
    g_print ("Capture queue & filter could not be created.\n");
    goto done;
  }

  gst_bin_add_many (GST_BIN (nvgstcameraParam.camera),
      nvgstUsbCamParam.usbcam_cap_queue, nvgstcameraParam.capturefilter, NULL);

  /* Manually link the Tee with capture queue */
  tee_vid_pad =
      gst_element_request_pad (nvgstUsbCamParam.usbcam_tee,
      tee_src_pad_template, NULL, NULL);
  sinkpad =
      gst_element_get_static_pad (nvgstUsbCamParam.usbcam_cap_queue, "sink");
  if (!sinkpad || !tee_vid_pad) {
    goto done;
  }
  if (GST_PAD_LINK_OK != gst_pad_link (tee_vid_pad, sinkpad)) {
    goto done;
  }
  gst_object_unref (sinkpad);
  gst_object_unref (tee_vid_pad);

  /* Add buffer probe on capture queue sink pad */
  sinkpad =
      gst_element_get_static_pad (nvgstUsbCamParam.usbcam_cap_queue, "sink");
  nvgstUsbCamParam.handler_id =
      gst_pad_add_buffer_probe (sinkpad, G_CALLBACK (encoder_buf_prob), NULL);
  gst_object_unref (sinkpad);

  if (nvgstcameraParam.mode == CAPTURE_IMAGE) {
    /* Create image encode chain elements */
    nvgstcameraParam.image_enc = NVGST_DEFAULT_USBCAM_IMAGE_ENCODER;
    if (!get_image_encoder (&nvgstUsbCamParam.encoder)) {
      g_print ("Image encoder element could be created.\n");
      goto done;
    }

    image_sink = gst_element_factory_make ("fakesink", NULL);
    if (!image_sink)
      goto done;
    g_object_set (G_OBJECT (image_sink), "signal-handoffs", TRUE, NULL);
    g_signal_connect (G_OBJECT (image_sink), "handoff",
        G_CALLBACK (USBCam_image_captured), NULL);

    gst_bin_add_many (GST_BIN (nvgstcameraParam.camera),
        nvgstUsbCamParam.encoder, image_sink, NULL);

    if ((gst_element_link (nvgstUsbCamParam.usbcam_cap_queue,
                nvgstUsbCamParam.encoder)) != TRUE) {
      g_print ("Elements could not link usbcam_cap_queue & iencoder\n");
      goto done;
    }

    if (gst_element_link (nvgstUsbCamParam.encoder, image_sink) != TRUE) {
      g_print ("iencoder & image_sink linking failed\n");
      goto done;
    }
  } else if (nvgstcameraParam.mode == CAPTURE_VIDEO) {
    /* Create video encode chain elements */
    nvgstcameraParam.video_sink = gst_element_factory_make ("filesink", NULL);

    if (!get_video_encoder (&nvgstUsbCamParam.encoder)) {
      g_print ("Video encoder element could not be created.\n");
      goto done;
    }

    if (!get_muxer (&nvgstcameraParam.muxer)) {
      g_print ("Video muxer element could not be created.\n");
      goto done;
    }

    if (!nvgstcameraParam.video_sink) {
      g_print ("video_sink element could not be created.\n");
      goto done;
    }

    g_object_set (G_OBJECT (nvgstcameraParam.video_sink), "location",
        DEFAULT_LOCATION, NULL);
    g_object_set (G_OBJECT (nvgstcameraParam.video_sink), "async", FALSE, NULL);
    g_object_set (G_OBJECT (nvgstcameraParam.video_sink), "sync", FALSE, NULL);

    gst_bin_add_many (GST_BIN (nvgstcameraParam.camera),
        nvgstUsbCamParam.encoder,
        nvgstcameraParam.muxer, nvgstcameraParam.video_sink, NULL);

    if (gst_element_link (nvgstUsbCamParam.usbcam_cap_queue,
            nvgstUsbCamParam.encoder) != TRUE
        || gst_element_link (nvgstcameraParam.muxer,
            nvgstcameraParam.video_sink) != TRUE) {
      g_print ("Video encode Chain elements could not be linked2\n");
      goto done;
    }

    srcpad = gst_element_get_static_pad (nvgstUsbCamParam.encoder, "src");
    sinkpad = gst_element_get_request_pad (nvgstcameraParam.muxer, "video_%d");
    if (!sinkpad || !srcpad) {
      goto done;
    }
    if (GST_PAD_LINK_OK != gst_pad_link (srcpad, sinkpad)) {
      goto done;
    }
    gst_object_unref (sinkpad);
    gst_object_unref (srcpad);
  }

  if (GST_STATE_CHANGE_FAILURE ==
      gst_element_set_state (nvgstcameraParam.camera, GST_STATE_PLAYING)) {
    g_warning ("can't set usb cam pipeline to playing\n");
    goto done;
  }
  return TRUE;

done:
  nvgstcameraParam.return_value = -1;
  g_print ("********* DETECTED A PROBLEM WITH YUV USB CAM NATIVE PIPELINE\n");
  return FALSE;
}

static gboolean
create_native_usbcam_pipeline (void)
{
  gboolean ret = TRUE;

  if ((nvgstcameraParam.preview_width < 176) ||
      (nvgstcameraParam.preview_height < 144) ||
      (nvgstcameraParam.video_enc < FORMAT_H264_HW) ||
      (nvgstcameraParam.image_enc < 0) || (nvgstcameraParam.mode < 0)) {
    g_print ("Invalid input parameters\n");
  }

  switch (nvgstUsbCamParam.usbcam_type) {
    case YUV:
      ret = create_yuv_USBcam_pipeline ();
      break;

    case MJPEG:
      ret = create_mjpeg_USBcam_pipeline ();
      break;

    default:
      g_print ("USB cam type not supported\n");
      ret = FALSE;
      break;
  }

  if (ret != TRUE)
    return FALSE;

  return TRUE;
}

static gboolean
create_native_pipeline (void)
{
  GstElement *previewfilter;
  GstElement *image_sink;
  GstPad *srcpad = NULL;
  GstPad *sinkpad = NULL;
  GstCaps *caps, *tempcaps;
  GstBus *bus;

  nvgstcameraParam.camera = gst_pipeline_new ("capture");
  if (!nvgstcameraParam.camera)
    goto done;

  bus = gst_pipeline_get_bus (GST_PIPELINE (nvgstcameraParam.camera));
  gst_bus_set_sync_handler (bus, NULL, NULL);
  gst_bus_set_sync_handler (bus, bus_sync_handler, nvgstcameraParam.camera);
  gst_bus_add_watch (bus, bus_call, NULL);
  gst_object_unref (bus);

  previewfilter = gst_element_factory_make ("capsfilter", NULL);
  if (!previewfilter)
    goto done;

  caps = gst_caps_new_simple (nvgstcameraParam.preview_mimetype,
      "width", G_TYPE_INT, nvgstcameraParam.preview_width,
      "height", G_TYPE_INT, nvgstcameraParam.preview_height, NULL);

  tempcaps = gst_caps_new_simple (nvgstcameraParam.preview_mimetype,
      "width", G_TYPE_INT, nvgstcameraParam.preview_height,
      "height", G_TYPE_INT, nvgstcameraParam.preview_width, NULL);

  gst_caps_append (caps, tempcaps);

  g_object_set (G_OBJECT (previewfilter), "caps", caps, NULL);
  gst_caps_unref (caps);

  g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "mode", nvgstcameraParam.mode,
      NULL);
  gst_bin_add_many (GST_BIN (nvgstcameraParam.camera), nvgstcameraParam.vsrc,
      previewfilter, nvgstcameraParam.vsink, NULL);

  if (!g_strcmp0 ("ximagesink", nvgstcameraParam.svs)) {
    nvgstcameraParam.colorspace_conv =
        gst_element_factory_make ("ffmpegcolorspace", NULL);
    if (!nvgstcameraParam.colorspace_conv)
      goto done;

    gst_bin_add_many (GST_BIN (nvgstcameraParam.camera),
        nvgstcameraParam.colorspace_conv, NULL);

    gst_element_link_many (nvgstcameraParam.colorspace_conv,
        nvgstcameraParam.vsink, NULL);

    srcpad = gst_element_get_static_pad (previewfilter, "src");
    sinkpad =
        gst_element_get_static_pad (nvgstcameraParam.colorspace_conv, "sink");

    if (!sinkpad || !srcpad)
      goto done;
    gst_pad_link (srcpad, sinkpad);
    gst_object_unref (sinkpad);
    gst_object_unref (srcpad);

  } else {
    gst_element_link_many (previewfilter, nvgstcameraParam.vsink, NULL);
  }

  srcpad = gst_element_get_static_pad (nvgstcameraParam.vsrc, "vfsrc");
  sinkpad = gst_element_get_static_pad (previewfilter, "sink");

  if (!sinkpad || !srcpad)
    goto done;
  gst_pad_link (srcpad, sinkpad);
  gst_object_unref (sinkpad);
  gst_object_unref (srcpad);

  caps = gst_caps_new_simple (nvgstcameraParam.capture_mimetype,
      "width", G_TYPE_INT, nvgstcameraParam.image_capture_width,
      "height", G_TYPE_INT, nvgstcameraParam.image_capture_height, NULL);

  nvgstcameraParam.image_filter = gst_element_factory_make ("capsfilter", NULL);
  if (!nvgstcameraParam.image_filter)
    goto done;
  g_object_set (G_OBJECT (nvgstcameraParam.image_filter), "caps", caps, NULL);
  gst_caps_unref (caps);

  if (!get_image_encoder (&nvgstcameraParam.iencoder))
    goto done;

  image_sink = gst_element_factory_make ("fakesink", NULL);
  if (!image_sink)
    goto done;
  g_object_set (G_OBJECT (image_sink), "signal-handoffs", TRUE, NULL);
  g_signal_connect (G_OBJECT (image_sink), "handoff",
      G_CALLBACK (image_captured), NULL);

  gst_bin_add_many (GST_BIN (nvgstcameraParam.camera),
      nvgstcameraParam.image_filter, nvgstcameraParam.iencoder, image_sink,
      NULL);

  gst_element_link_many (nvgstcameraParam.image_filter,
      nvgstcameraParam.iencoder, image_sink, NULL);

  srcpad = gst_element_get_static_pad (nvgstcameraParam.vsrc, "imgsrc");
  sinkpad = gst_element_get_static_pad (nvgstcameraParam.image_filter, "sink");
  if (!sinkpad || !srcpad)
    goto done;
  gst_pad_link (srcpad, sinkpad);
  gst_object_unref (sinkpad);
  gst_object_unref (srcpad);

  nvgstcameraParam.video_sink = gst_element_factory_make ("filesink", NULL);
  if (!nvgstcameraParam.video_sink)
    goto done;
  g_object_set (G_OBJECT (nvgstcameraParam.video_sink), "async", FALSE, NULL);
  g_object_set (G_OBJECT (nvgstcameraParam.video_sink), "sync", FALSE, NULL);

  gst_element_set_locked_state (nvgstcameraParam.video_sink, TRUE);
  g_object_set (nvgstcameraParam.video_sink, "location", DEFAULT_LOCATION,
      NULL);

  caps = gst_caps_new_simple (nvgstcameraParam.capture_mimetype,
      "width", G_TYPE_INT, nvgstcameraParam.video_capture_width,
      "height", G_TYPE_INT, nvgstcameraParam.video_capture_height, NULL);

  nvgstcameraParam.capturefilter =
      gst_element_factory_make ("capsfilter", NULL);
  if (!nvgstcameraParam.capturefilter)
    goto done;
  g_object_set (G_OBJECT (nvgstcameraParam.capturefilter), "caps", caps, NULL);
  gst_caps_unref (caps);

  if (!get_video_encoder (&nvgstcameraParam.vencoder))
    goto done;

  if (nvgstcameraParam.video_enc == FORMAT_H264_HW) {
    GstCaps *h264enc_caps = NULL;

    /* h264_filter and h264parse are necessary to make byte-stream as default format for h264encoder
     * since avc support is broken on t11x BUG: 1180132*/

    //height should be multiple of 16
    nvgstcameraParam.video_capture_height =
        GST_ROUND_UP_16 (nvgstcameraParam.video_capture_height);
    nvgstcameraParam.h264_filter =
        gst_element_factory_make ("capsfilter", NULL);
    h264enc_caps =
        gst_caps_new_simple ("video/x-h264", "width", G_TYPE_INT,
        nvgstcameraParam.video_capture_width, "height", G_TYPE_INT,
        nvgstcameraParam.video_capture_height, "stream-format", G_TYPE_STRING,
        "byte-stream", "alignment", G_TYPE_STRING, "au", NULL);
    g_object_set (G_OBJECT (nvgstcameraParam.h264_filter), "caps", h264enc_caps,
        NULL);
    if (h264enc_caps)
      gst_caps_unref (h264enc_caps);
    nvgstcameraParam.h264parse = gst_element_factory_make ("h264parse", NULL);
    if (!nvgstcameraParam.h264parse)
      goto done;
  }

  if (!get_muxer (&nvgstcameraParam.muxer))
    goto done;

  if (nvgstcameraParam.Voice_Control != VOICE_OFF) {
    if (!get_audio_encoder (&nvgstcameraParam.aencoder))
      goto done;

    nvgstcameraParam.audio_src = gst_element_factory_make ("alsasrc", NULL);
    if (!nvgstcameraParam.audio_src)
      goto done;

    gst_element_set_locked_state (nvgstcameraParam.audio_src, TRUE);

    nvgstcameraParam.audiofilter =
        gst_element_factory_make ("capsfilter", NULL);
    if (!nvgstcameraParam.audiofilter)
      goto done;

    if (nvgstcameraParam.audio_enc == 0) {
      caps = gst_caps_new_simple ("audio/x-raw-int",
          "width", G_TYPE_INT, 16,
          "depth", G_TYPE_INT, 16,
          "channels", G_TYPE_INT, nvgstcameraParam.channels,
          "rate", G_TYPE_INT, nvgstcameraParam.sample_rate,
          "endianness", G_TYPE_INT, 1234, "signed", G_TYPE_BOOLEAN, TRUE, NULL);

    } else if (nvgstcameraParam.audio_enc == 1) {
      caps = gst_caps_new_simple ("audio/x-raw-int",
          "width", G_TYPE_INT, 16,
          "depth", G_TYPE_INT, 16,
          "channels", G_TYPE_INT, 1,
          "rate", G_TYPE_INT, 8000,
          "endianness", G_TYPE_INT, 1234, "signed", G_TYPE_BOOLEAN, TRUE, NULL);
    }
    g_object_set (G_OBJECT (nvgstcameraParam.audiofilter), "caps", caps, NULL);
    gst_caps_unref (caps);

    srcpad = gst_element_get_static_pad (nvgstcameraParam.audio_src, "src");
    gst_pad_add_data_probe (srcpad, (GCallback) data_probe, NULL);
    gst_object_unref (srcpad);

    gst_bin_add_many (GST_BIN (nvgstcameraParam.camera),
        nvgstcameraParam.audio_src, nvgstcameraParam.audiofilter,
        nvgstcameraParam.aencoder, NULL);

    gst_element_link_many (nvgstcameraParam.audio_src,
        nvgstcameraParam.audiofilter, nvgstcameraParam.aencoder, NULL);

  }
  if (nvgstcameraParam.video_enc == FORMAT_H264_HW) {
    gst_bin_add_many (GST_BIN (nvgstcameraParam.camera),
        nvgstcameraParam.capturefilter, nvgstcameraParam.vencoder,
        nvgstcameraParam.h264_filter, nvgstcameraParam.h264parse,
        nvgstcameraParam.muxer, nvgstcameraParam.video_sink, NULL);
  } else {
    gst_bin_add_many (GST_BIN (nvgstcameraParam.camera),
        nvgstcameraParam.capturefilter, nvgstcameraParam.vencoder,
        nvgstcameraParam.muxer, nvgstcameraParam.video_sink, NULL);
  }

  if (nvgstcameraParam.video_enc == FORMAT_H264_HW) {
    gst_element_link_many (nvgstcameraParam.vencoder,
        nvgstcameraParam.h264_filter, nvgstcameraParam.h264parse, NULL);
  }

  gst_element_link (nvgstcameraParam.capturefilter, nvgstcameraParam.vencoder);

  srcpad = gst_element_get_static_pad (nvgstcameraParam.vsrc, "vidsrc");
  sinkpad = gst_element_get_static_pad (nvgstcameraParam.capturefilter, "sink");

  if (!sinkpad || !srcpad)
    goto done;
  gst_pad_link (srcpad, sinkpad);
  gst_object_unref (sinkpad);
  gst_object_unref (srcpad);

  gst_element_link (nvgstcameraParam.muxer, nvgstcameraParam.video_sink);

  if (nvgstcameraParam.video_enc == FORMAT_H264_HW) {
    gst_element_link (nvgstcameraParam.h264parse, nvgstcameraParam.muxer);

  } else {
    sinkpad = gst_element_get_request_pad (nvgstcameraParam.muxer, "video_%d");
    srcpad = gst_element_get_static_pad (nvgstcameraParam.vencoder, "src");
    if (!sinkpad || !srcpad)
      goto done;
    gst_pad_link (srcpad, sinkpad);
    gst_object_unref (sinkpad);
    gst_object_unref (srcpad);
  }

  if (VOICE_OFF != nvgstcameraParam.Voice_Control) {
    sinkpad = gst_element_get_request_pad (nvgstcameraParam.muxer, "audio_%d");
    srcpad = gst_element_get_static_pad (nvgstcameraParam.aencoder, "src");
    if (!sinkpad || !srcpad)
      goto done;
    gst_pad_link (srcpad, sinkpad);
    gst_object_unref (sinkpad);
    gst_object_unref (srcpad);
  }

  if (GST_STATE_CHANGE_FAILURE ==
      gst_element_set_state (nvgstcameraParam.camera, GST_STATE_PLAYING)) {
    g_warning ("can't set pipeline to playing\n");
    goto done;
  }
  return TRUE;

done:
  nvgstcameraParam.return_value = -1;
  g_print ("*************************** DETECTED A PROBLEM\n");
  return FALSE;
}

#ifdef GST_KPI_MEASURE
static gboolean
launch_time_probe (GstPad * pad, GstBuffer * buf, gpointer data)
{
  if (nvgstcameraParam.timer) {
    g_timer_stop (nvgstcameraParam.timer);
    g_print ("**** LAUNCH TIME: %g secs\n",
        g_timer_elapsed (nvgstcameraParam.timer, NULL));
    g_timer_destroy (nvgstcameraParam.timer);

    nvgstcameraParam.timer = NULL;
  }

  return TRUE;
}
#endif

static gboolean
create_pipeline (void)
{
  GstCaps *caps;
  GstBus *bus;
#ifdef GST_KPI_MEASURE
  GstPad *wpad;
#endif
  gint flags;

  if (nvgstcameraParam.image_capture_width < 176)
    goto done;
  if (nvgstcameraParam.image_capture_width < 144)
    goto done;
  if (nvgstcameraParam.preview_width < 176)
    goto done;
  if (nvgstcameraParam.preview_height < 144)
    goto done;
  if (nvgstcameraParam.video_enc < FORMAT_H264_HW)
    goto done;
  if (nvgstcameraParam.audio_enc < 0)
    goto done;
  if (nvgstcameraParam.image_enc < 0)
    goto done;
  if (nvgstcameraParam.mode < 0)
    goto done;

  nvgstcameraParam.vsrc = gst_element_factory_make ("nv_omx_camera2", NULL);
  if (!nvgstcameraParam.vsrc)
    goto done;

#ifdef GST_KPI_MEASURE
  if (nvgstcameraParam.perf_measure) {
    wpad = gst_element_get_static_pad (nvgstcameraParam.vsrc, "vfsrc");
    gst_pad_add_buffer_probe (wpad, G_CALLBACK (launch_time_probe), NULL);
    gst_object_unref (wpad);
  }
#endif

  g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "sensor",
      nvgstcameraParam.sensorid, NULL);
  g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "stereo-mode",
      nvgstcameraParam.stereomode, NULL);
  g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "contrast",
      nvgstcameraParam.contrast, NULL);
  g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "brightness",
      nvgstcameraParam.brightness, NULL);
  g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "saturation",
      nvgstcameraParam.saturation, NULL);
  g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "hue", nvgstcameraParam.hue,
      NULL);
  g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "exposure",
      nvgstcameraParam.exposure, NULL);
  g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "flicker-mode",
      nvgstcameraParam.flicker, NULL);
  g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "edge-enhancement",
      nvgstcameraParam.edgeenhancement, NULL);
  g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "colour-tone-mode",
      nvgstcameraParam.imagefilter, NULL);
  g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "white-balance-mode",
      nvgstcameraParam.whitebalance, NULL);
  g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "framerate",
      nvgstcameraParam.fps_rate, NULL);
  g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "flash-mode",
      nvgstcameraParam.flash_mode, NULL);
#ifdef GST_KPI_MEASURE
  g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "enable-kpi-measure",
      nvgstcameraParam.perf_measure, NULL);
#endif

  nvgstcameraParam.vsink =
      gst_element_factory_make (nvgstcameraParam.svs, NULL);
  if (!nvgstcameraParam.vsink)
    goto done;
  g_object_set (G_OBJECT (nvgstcameraParam.vsink), "async", FALSE, NULL);
  g_object_set (G_OBJECT (nvgstcameraParam.vsink), "sync", FALSE, NULL);

#ifdef USE_CAMERABIN2
  if (nvgstcameraParam.use_camerabin) {
    nvgstcameraParam.camera = gst_element_factory_make ("camerabin2", NULL);
    if (!nvgstcameraParam.camera)
      goto done;

    bus = gst_pipeline_get_bus (GST_PIPELINE (nvgstcameraParam.camera));
    gst_bus_set_sync_handler (bus, NULL, NULL);
    gst_bus_set_sync_handler (bus, bus_sync_handler, nvgstcameraParam.camera);
    gst_bus_add_watch (bus, bus_call, NULL);
    gst_object_unref (bus);

    g_object_set (G_OBJECT (nvgstcameraParam.camera), "camera-source",
        nvgstcameraParam.vsrc, NULL);
    g_object_set (G_OBJECT (nvgstcameraParam.camera), "mode",
        nvgstcameraParam.mode, NULL);

    if (nvgstcameraParam.Voice_Control != VOICE_OFF) {
      nvgstcameraParam.audio_src =
          gst_element_factory_make ("alsasrc", "audiosrc");
      if (!nvgstcameraParam.audio_src)
        goto done;
      g_object_set (G_OBJECT (nvgstcameraParam.camera), "audio-source",
          nvgstcameraParam.audio_src, NULL);
    }

    flags = 0x00000001 | 0x00000002 | 0x00000004 | 0x00000008;
    g_object_set (G_OBJECT (nvgstcameraParam.camera), "flags", flags, NULL);

    g_object_set (G_OBJECT (nvgstcameraParam.camera), "viewfinder-sink",
        nvgstcameraParam.vsink, NULL);

    GstEncodingProfile *prof;
    prof = create_image_profile (nvgstcameraParam.image_enc);
    if (!prof)
      goto done;
    g_object_set (G_OBJECT (nvgstcameraParam.camera), "image-profile", prof,
        NULL);

    prof =
        create_video_profile (nvgstcameraParam.file_type,
        nvgstcameraParam.video_enc, nvgstcameraParam.audio_enc);
    if (!prof)
      goto done;
    g_object_set (G_OBJECT (nvgstcameraParam.camera), "video-profile", prof,
        NULL);

    caps = gst_caps_new_simple ("video/x-raw-yuv",
        "format", GST_TYPE_FOURCC, GST_MAKE_FOURCC ('N', 'V', 'B', '1'),
        "width", G_TYPE_INT, nvgstcameraParam.preview_width,
        "height", G_TYPE_INT, nvgstcameraParam.preview_height, NULL);

    g_object_set (G_OBJECT (nvgstcameraParam.camera), "viewfinder-caps", caps,
        NULL);
    gst_caps_unref (caps);

    caps = gst_caps_new_simple (nvgstcameraParam.mimetype,
        "width", G_TYPE_INT, nvgstcameraParam.image_capture_width,
        "height", G_TYPE_INT, nvgstcameraParam.image_capture_height, NULL);

    g_object_set (G_OBJECT (nvgstcameraParam.camera), "image-capture-caps",
        caps, NULL);
    gst_caps_unref (caps);

    caps = gst_caps_new_simple (nvgstcameraParam.mimetype,
        "width", G_TYPE_INT, nvgstcameraParam.video_capture_width,
        "height", G_TYPE_INT, nvgstcameraParam.video_capture_height, NULL);

    g_object_set (G_OBJECT (nvgstcameraParam.camera), "video-capture-caps",
        caps, NULL);
    gst_caps_unref (caps);

    GST_INFO_OBJECT (nvgstcameraParam.camera, "camerabin ready");
    if (GST_STATE_CHANGE_FAILURE ==
        gst_element_set_state (nvgstcameraParam.camera, GST_STATE_PLAYING)) {
      g_warning ("can't set camerabin to playing\n");
      goto done;
    }
    return TRUE;
  } else
#endif
  {
    if (nvgstcameraParam.raw_prev_cap) {
      nvgstcameraParam.mimetype = "video/x-raw-yuv";
      nvgstcameraParam.preview_mimetype = "video/x-raw-yuv";
      nvgstcameraParam.capture_mimetype = "video/x-raw-yuv";
    } else if (nvgstcameraParam.raw_preview) {
      nvgstcameraParam.preview_mimetype = "video/x-raw-yuv";
    } else if (nvgstcameraParam.raw_capture) {
      nvgstcameraParam.capture_mimetype = "video/x-raw-yuv";
    }

    return create_native_pipeline ();
  }
done:
  nvgstcameraParam.return_value = -1;
  g_print ("*************************** DETECTED A PROBLEM\n");
  return FALSE;
}

static gboolean
create_vip_pipeline (void)
{
  GstElement *captureelement;
  GstBus *bus;
  GstElement *bin;

  captureelement = gst_element_factory_make ("nv_omx_vip", "camera-source");

  g_object_set (G_OBJECT (captureelement), "format", nvgstVipParam.formattype,
      NULL);
  g_object_set (G_OBJECT (captureelement), "frame", nvgstVipParam.frametype,
      NULL);

  nvgstcameraParam.vsink =
      gst_element_factory_make ("nv_gstbin_videosink", "video-output");
  g_object_set (G_OBJECT (nvgstcameraParam.vsink), "rendertarget",
      nvgstVipParam.rendertarget, NULL);

  bin = gst_bin_new ("capturebin");
  gst_bin_add_many (GST_BIN (bin), captureelement, nvgstcameraParam.vsink,
      NULL);
  gst_element_link (captureelement, nvgstcameraParam.vsink);

  nvgstcameraParam.camera = gst_pipeline_new ("pipeline");
  gst_bin_add (GST_BIN (nvgstcameraParam.camera), bin);

  bus = gst_pipeline_get_bus (GST_PIPELINE (nvgstcameraParam.camera));
  gst_bus_add_watch (bus, bus_call, NULL);
  gst_object_unref (bus);

  if (GST_STATE_CHANGE_FAILURE ==
      gst_element_set_state (nvgstcameraParam.camera, GST_STATE_PLAYING)) {
    nvgstcameraParam.return_value = -1;
    g_warning ("Error: Can't set pipeline to playing\n");
    return FALSE;
  }

  return TRUE;
}

static void
destroy_vip_pipeline (void)
{
  gst_element_set_state (nvgstcameraParam.camera, GST_STATE_READY);
  gst_element_set_state (nvgstcameraParam.camera, GST_STATE_NULL);
  nvgstcameraParam.camera = NULL;
  nvgstcameraParam.vsink = NULL;
}

static gboolean
create_CSIcam_pipeline ()
{
  gboolean ret = TRUE;

  ret = create_yuv_USBcam_pipeline ();

  return ret;
}

static gboolean
create_usbcam_pipeline ()
{
  GstCaps *caps;
  GstBus *bus;
  gint flags;
  GstElement *cam_wrapper_src;
  GstElement *svsbin;
  GstPad *pad;

#ifdef USE_CAMERABIN2

  if ((nvgstcameraParam.preview_width < 176) ||
      (nvgstcameraParam.preview_height < 144) ||
      (nvgstcameraParam.video_enc < FORMAT_H264_HW) ||
      (nvgstcameraParam.audio_enc < 0) ||
      (nvgstcameraParam.image_enc < 0) || (nvgstcameraParam.mode < 0))
    goto done;

  if (nvgstcameraParam.use_camerabin) {
    nvgstcameraParam.camera = gst_element_factory_make ("camerabin2", NULL);
    if (!nvgstcameraParam.camera)
      goto done;

    bus = gst_pipeline_get_bus (GST_PIPELINE (nvgstcameraParam.camera));
    gst_bus_set_sync_handler (bus, NULL, NULL);
    gst_bus_set_sync_handler (bus, bus_sync_handler, nvgstcameraParam.camera);
    gst_bus_add_watch (bus, bus_call, NULL);
    gst_object_unref (bus);

    /* Create the sink element */
    nvgstcameraParam.vsink =
        gst_element_factory_make (nvgstcameraParam.svs, NULL);
    if (!nvgstcameraParam.vsink)
      goto done;
    g_object_set (G_OBJECT (nvgstcameraParam.vsink), "async", FALSE, NULL);
    g_object_set (G_OBJECT (nvgstcameraParam.vsink), "sync", FALSE, NULL);

    if (!g_strcmp0 ("ximagesink", nvgstcameraParam.svs)) {
      nvgstcameraParam.colorspace_conv =
          gst_element_factory_make ("ffmpegcolorspace", NULL);
    } else if (!g_strcmp0 ("xvimagesink", nvgstcameraParam.svs)) {
      nvgstcameraParam.colorspace_conv =
          gst_element_factory_make ("ffmpegcolorspace", NULL);
    } else {
      g_print ("video sink not supported\n");
      goto done;
    }
    if (!nvgstcameraParam.colorspace_conv)
      goto done;

    svsbin = gst_bin_new ("svs_bin");
    gst_bin_add_many (GST_BIN (svsbin), nvgstcameraParam.colorspace_conv,
        nvgstcameraParam.vsink, NULL);
    gst_element_link (nvgstcameraParam.colorspace_conv, nvgstcameraParam.vsink);

    pad = gst_element_get_static_pad (nvgstcameraParam.colorspace_conv, "sink");
    gst_element_add_pad (svsbin, gst_ghost_pad_new ("sink", pad));
    gst_object_unref (GST_OBJECT (pad));

    g_object_set (G_OBJECT (nvgstcameraParam.camera), "viewfinder-sink", svsbin,
        NULL);

    /* Create the source element */
    nvgstcameraParam.vsrc = gst_element_factory_make ("v4l2src", NULL);
    g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "device",
        nvgstUsbCamParam.vidcap_device, NULL);
    if (!nvgstcameraParam.vsrc)
      goto done;

    cam_wrapper_src = gst_element_factory_make ("wrappercamerabinsrc", NULL);
    if (!cam_wrapper_src)
      goto done;
    g_object_set (cam_wrapper_src, "post-previews", FALSE, NULL);

    g_object_set (cam_wrapper_src, "video-source", nvgstcameraParam.vsrc, NULL);
    g_object_set (nvgstcameraParam.camera, "camera-source", cam_wrapper_src,
        "post-previews", FALSE, NULL);
    g_object_set (nvgstcameraParam.camera, "mode", nvgstcameraParam.mode, NULL);

    if (nvgstcameraParam.Voice_Control != VOICE_OFF) {
      nvgstcameraParam.audio_src =
          gst_element_factory_make ("alsasrc", "audiosrc");
      if (!nvgstcameraParam.audio_src)
        goto done;
      g_object_set (G_OBJECT (nvgstcameraParam.camera), "audio-source",
          nvgstcameraParam.audio_src, NULL);

      caps = gst_caps_new_simple ("audio/x-raw-int",
          "signed", G_TYPE_BOOLEAN, TRUE,
          "endianness", G_TYPE_INT, 1234,
          "width", G_TYPE_INT, 16,
          "depth", G_TYPE_INT, 16,
          "rate", G_TYPE_INT, 44100, "channels", G_TYPE_INT, 2, NULL);
      g_object_set (nvgstcameraParam.camera, "audio-capture-caps", caps, NULL);
      gst_caps_unref (caps);
    }

    flags = 0x00000001 | 0x00000002 | 0x00000004 | 0x00000008;
    g_object_set (G_OBJECT (nvgstcameraParam.camera), "flags", flags, NULL);

    GstEncodingProfile *prof;
    prof = create_image_profile (nvgstcameraParam.image_enc);
    if (!prof)
      goto done;
    g_object_set (G_OBJECT (nvgstcameraParam.camera), "image-profile", prof,
        NULL);

    prof =
        create_video_profile (nvgstcameraParam.file_type,
        nvgstcameraParam.video_enc, nvgstcameraParam.audio_enc);
    if (!prof)
      goto done;
    g_object_set (G_OBJECT (nvgstcameraParam.camera), "video-profile", prof,
        NULL);

    caps = gst_caps_new_simple ("video/x-raw-yuv",
        "format", GST_TYPE_FOURCC, GST_MAKE_FOURCC ('I', '4', '2', '0'),
        "width", G_TYPE_INT, nvgstcameraParam.preview_width,
        "height", G_TYPE_INT, nvgstcameraParam.preview_height, NULL);

    g_object_set (nvgstcameraParam.camera,
        "viewfinder-caps", caps,
        "image-capture-caps", caps, "video-capture-caps", caps, NULL);
    gst_caps_unref (caps);

    GST_INFO_OBJECT (nvgstcameraParam.camera, "camerabin ready");
    if (GST_STATE_CHANGE_FAILURE ==
        gst_element_set_state (nvgstcameraParam.camera, GST_STATE_PLAYING)) {
      g_warning ("can't set camerabin to playing for camerabin 2 case\n");
      goto done;
    }
    return TRUE;

  done:
    nvgstcameraParam.return_value = -1;
    g_print ("********* DETECTED A PROBLEM WITH USB CAM CAMERABIN PIPELINE\n");
    return FALSE;

  } else
#endif
  {
    return create_native_usbcam_pipeline ();
  }
}

static void
print_help (void)
{
  printf ("\n\nCamera Runtime Options\n\n");
  g_print ("%s\n", nvgstcameraParam.extra_options);
}

static gboolean
exit_capture (gpointer data)
{
  g_main_loop_quit (loop);

  return FALSE;
}

static gboolean
auto_capture (gpointer data)
{
  gboolean ret = FALSE;
  gint sensor_count = nvgstcameraParam.iteration_count;
  gint resolutions_widths[NUMBER_OF_RESOLUTIONS] =
      { 1920, 1280, 720, 640, 352, 320, 176 };
  gint resolutions_heights[NUMBER_OF_RESOLUTIONS] =
      { 1088, 720, 480, 480, 288, 240, 144 };

  g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "zoom", nvgstcameraParam.zoom,
      NULL);
  g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "metering-mode",
      nvgstcameraParam.metermode, NULL);

  if (nvgstcameraParam.toggle_mode) {
    while (nvgstcameraParam.iteration_count-- > 0) {
      g_usleep (3000000);       //time the preview is in original mode
      nvgstcameraParam.mode = 3 - nvgstcameraParam.mode;        //toggle between still and video mode
      g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "mode",
          nvgstcameraParam.mode, NULL);
      g_usleep (3000000);       //time the preview is in second mode before next iteration/capture
    }
  }

  if (nvgstcameraParam.toggle_autofocus) {
    while (nvgstcameraParam.iteration_count-- > 0) {
      g_signal_emit_by_name (G_OBJECT (nvgstcameraParam.vsrc),
          "start-auto-focus", TRUE, TRUE, TRUE, &ret);
      g_usleep (5000000);       //time is required for autofocus to start
      g_signal_emit_by_name (G_OBJECT (nvgstcameraParam.vsrc),
          "stop-auto-focus", TRUE, TRUE, TRUE, &ret);
      g_usleep (5000000);       //time is required for autofocus to stop
    }
  }

  if (nvgstcameraParam.all_resolution_images
      || nvgstcameraParam.all_resolution_videos)
    nvgstcameraParam.iteration_count = NUMBER_OF_RESOLUTIONS;

  while (nvgstcameraParam.iteration_count-- > 0) {
    if (nvgstcameraParam.automate_capture) {
      if (nvgstcameraParam.mode == CAPTURE_IMAGE && recording == FALSE) {
        g_object_set (G_OBJECT (nvgstcameraParam.vsrc), "pause-after-capture",
            TRUE, NULL);

        if (nvgstcameraParam.sensorid != 1) {
          g_signal_emit_by_name (G_OBJECT (nvgstcameraParam.vsrc),
              "start-auto-focus", TRUE, TRUE, TRUE, &ret);
        }

        if (nvgstcameraParam.all_resolution_images) {

          destroy_pipeline ();
          nvgstcameraParam.image_capture_width =
              resolutions_widths[nvgstcameraParam.iteration_count];
          nvgstcameraParam.image_capture_height =
              resolutions_heights[nvgstcameraParam.iteration_count];
          g_print ("w = %d, h = %d\n", nvgstcameraParam.image_capture_width,
              nvgstcameraParam.image_capture_height);
          create_pipeline ();

        } else if (nvgstcameraParam.all_resolution_videos)
          g_print
              ("\nWrong mode! Use '--mode=2' for 'all-resolution-videos'\n");

        recording = TRUE;

        if (nvgstcameraParam.use_camerabin) {
          set_new_image_file_name (0);
          g_signal_emit_by_name (G_OBJECT (nvgstcameraParam.camera),
              "start-capture", &ret);
        } else
          g_signal_emit_by_name (G_OBJECT (nvgstcameraParam.vsrc),
              "start-capture", &ret);

        g_mutex_lock (nvgstcameraParam.lock);
        while (recording) {
          g_cond_wait (nvgstcameraParam.cond, nvgstcameraParam.lock);
        }
        g_mutex_unlock (nvgstcameraParam.lock);

        gst_element_post_message (GST_ELEMENT (nvgstcameraParam.camera),
            gst_message_new_eos (GST_OBJECT (nvgstcameraParam.camera)));
        g_usleep (nvgstcameraParam.capture_gap * 1000000);      //time between successive image captures

      } else if (nvgstcameraParam.mode == CAPTURE_VIDEO && recording == FALSE) {

        if (nvgstcameraParam.all_resolution_videos) {
          destroy_pipeline ();
          nvgstcameraParam.video_capture_width =
              resolutions_widths[nvgstcameraParam.iteration_count];
          nvgstcameraParam.video_capture_height =
              resolutions_heights[nvgstcameraParam.iteration_count];
          g_print ("w = %d, h = %d\n", nvgstcameraParam.video_capture_width,
              nvgstcameraParam.video_capture_height);
          create_pipeline ();

        } else if (nvgstcameraParam.all_resolution_images) {
          g_print
              ("\nWrong mode! Use '--mode=1' (default) for 'all-resolution-images'\n");
        }

        set_new_video_file_name (nvgstcameraParam.file_type);
        recording = TRUE;

        if (nvgstcameraParam.use_camerabin) {
          g_signal_emit_by_name (G_OBJECT (nvgstcameraParam.camera),
              "start-capture", &ret);
        } else {
          if (nvgstcameraParam.audio_src) {
            GstClock *clock =
                gst_pipeline_get_clock (GST_PIPELINE_CAST
                (nvgstcameraParam.camera));

            gst_element_set_state (nvgstcameraParam.audio_src,
                GST_STATE_PAUSED);
            gst_element_set_base_time (nvgstcameraParam.audio_src,
                gst_element_get_base_time (GST_ELEMENT_CAST
                    (nvgstcameraParam.camera)));
            if (clock) {
              gst_element_set_clock (nvgstcameraParam.audio_src, clock);
              gst_object_unref (clock);
            }
          }
          g_signal_emit_by_name (G_OBJECT (nvgstcameraParam.vsrc),
              "start-capture", &ret);
          gst_element_set_locked_state (nvgstcameraParam.audio_src, FALSE);
          nvgstcameraParam.send_new_seg = TRUE;
          gst_element_set_state (nvgstcameraParam.audio_src, GST_STATE_PLAYING);
        }
        g_print ("\nRecording Started\n");

        /* wait till user specified record time (Default record time is 10 seconds) */
        g_usleep (nvgstcameraParam.capture_time * 1000000);

        {
          recording = FALSE;

          if (nvgstcameraParam.use_camerabin) {
            g_signal_emit_by_name (G_OBJECT (nvgstcameraParam.camera),
                "stop-capture", &ret);
          } else {
            g_signal_emit_by_name (G_OBJECT (nvgstcameraParam.vsrc),
                "stop-capture", &ret);
            if (VOICE_OFF != nvgstcameraParam.Voice_Control) {
              gst_element_send_event (nvgstcameraParam.audio_src,
                  gst_event_new_eos ());
            }
            sleep (1);
            if (!g_thread_create (reset_elements, NULL, FALSE, NULL)) {
              g_print ("Failed to create thread to "
                  "reset video elements' state, video recordings may not work "
                  "anymore");
            }
          }
          g_print ("Stopped Recording \n");
        }
        g_usleep (nvgstcameraParam.capture_gap * 1000000);      //time between successive video recordings
      }
    }
    if (nvgstcameraParam.toggle_sensor && nvgstcameraParam.iteration_count) {
      sensor_count = 1;
      while (sensor_count-- > 0) {
        destroy_pipeline ();
        nvgstcameraParam.sensorid = !nvgstcameraParam.sensorid;
        create_pipeline ();
        g_usleep (3000000);     //not actually required, just there so that user sees camera switching
      }
    }
  }

  g_timeout_add_seconds (nvgstcameraParam.quit_time, exit_capture, NULL);

  return FALSE;
}


static void
_intr_handler (int signum)
{
  struct sigaction action;

  g_print ("User Interrupted.. \n");
  nvgstcameraParam.return_value = -1;
  memset (&action, 0, sizeof (action));
  action.sa_handler = SIG_DFL;

  sigaction (SIGINT, &action, NULL);

  cintr = TRUE;
}


static gboolean
check_for_interrupt (gpointer data)
{
  if (cintr) {
    cintr = FALSE;

    gst_element_post_message (GST_ELEMENT (nvgstcameraParam.camera),
        gst_message_new_application (GST_OBJECT (nvgstcameraParam.camera),
            gst_structure_new ("NvGstAppInterrupt",
                "message", G_TYPE_STRING, "Pipeline interrupted", NULL)));

    return FALSE;
  }

  return TRUE;
}


static void
_intr_setup (void)
{
  struct sigaction action;

  memset (&action, 0, sizeof (action));
  action.sa_handler = _intr_handler;

  sigaction (SIGINT, &action, NULL);
}


int
main (int argc, char *argv[])
{
  GOptionContext *ctx;
  GOptionGroup *group;
  GError *error = NULL;

#ifndef _INTERACTIVE
  GIOChannel *channel = NULL;
#endif

  if (!g_thread_supported ())
    g_thread_init (NULL);

  g_type_init ();

  nvgstcameraParam.extra_options = g_strdup ("Runtime Commands:\n\n "
      "  HELP: '-h'\n"
      "  Quit : 'q'\n"
      "  Set Capture Mode:\n"
      "      mo:<val>\n"
      "          (2): video\n"
      "          (1): image\n"
      "  Get Capture Mode:\n"
      "      gmo\n"
      "  Capture: enter 'j' OR\n"
      "           followed by a timer (eg. jx5000, capture after 5 seconds) OR\n"
      "           followed by multishot count (eg. j:6, capture 6 images)\n"
      "           timer/multihot values are optional, capture defaults to single shot with timer=0s\n"
      "  Start Recording: enter '1'\n"
      "  Stop Recording: enter '0'\n"
      "  Force IDR frame on video encoder: enter 'f' \n"
      /*
         //#ifdef CSI_CAPTURE_SUPPORTED
         "  Set Zoom value:\n" \
         "      z:<val> eg z:1.5\n" \
         "  Get Zoom value:\n" \
         "      gz\n" \
         "  Set Flash:\n" \
         "      xx:<val>\n"   \
         "          (0) : auto\n" \
         "          (1) : off\n"    \
         "          (2) : on\n"   \
         "          (3) : fill-in(Not supported)\n"   \
         "          (4) : red-eye(Not supported)\n"\
         "          (5) : torch(Not supported)\n"\
         "  Get Flash:\n" \
         "      gxx\n"
         "  Rotation Preview:\n"  \
         "      px:<val>, val=0,90,180,270\n" \
         "  Rotation Capture:\n" \
         "      cx:<val>, val=0,90,180,270\n" \
         "  Set Exposure:\n"  \
         "      ex:<val>, Range val= 0 - 200, 0=auto\n"  \
         "  Get Exposure:\n" \
         "      gex\n"
         "  Set White Balance:\n" \
         "      wb:<val> \n" \
         "          (0) : auto\n" \
         "          (1) : daylight\n"
         "          (2) : cloudy\n" \
         "          (3) : sunset(Not supported)\n" \
         "          (4) : tungsten\n" \
         "          (5) : fluorescent\n"\
         "          (6) : shade\n" \
         "          (7) : incandescent\n" \
         "          (8) : flash\n" \
         "          (9) : horizon\n" \
         "          (10) : off\n"\
         "  Get White Balance:\n" \
         "      gwb\n"\
         "  Set ISO Mode: (should not be changed with auto exposure on)\n" \
         "      io:<val>, val= 100,200,400,800, 0=auto\n" \
         "  Get ISO Mode:\n" \
         "      gio\n"
         "  Set Metering: (should not be changed with auto exposure on)\n" \
         "      mt:<val>, \n" \
         "          (0) : average\n" \
         "          (1) : spot\n" \
         "          (2) : matrix\n" \
         "  Get Metering:\n" \
         "      gmt\n"
         "  Set EV Compensation:\n" \
         "      ev:<val>, Range = -2.5 - 2.5 \n" \
         "  Get EV Compensation:\n" \
         "      gev\n"
         "  Set Brightness:\n" \
         "      bt:<val>, Range val= 0 - 100 \n" \
         "  Get Brightness:\n" \
         "      gbt\n"
         "  Set Saturation:\n" \
         "      st:<val>, Range val= -100 - 100\n" \
         "  Get Saturation:\n" \
         "      gst\n"
         "  Set Contrast:\n" \
         "      ct:<val>, Range val= 0 - 100 \n" \
         "  Get Contrast:\n" \
         "      gct\n"
         "  Set Hue:\n" \
         "      ht:<val>, Range val= 0 - 100\n" \
         "  Get Hue:\n" \
         "      ght\n"
         "  Set Flicker Control:\n" \
         "      fl:<val> \n" \
         "          (0) : off\n"   \
         "          (1) : 50Hz\n" \
         "          (2) : 60Hz\n" \
         "          (3) : auto\n" \
         "  Get Flicker Control:\n" \
         "      gfl\n"
         "  Set colour-tone-mode Control:\n" \
         "      ft:<val> \n" \
         "          (0) : normal\n" \
         "          (1) : sepia\n" \
         "          (2) : negative\n" \
         "          (3) : grayscale(Not supported)\n" \
         "          (4) : natural(Not supported)\n" \
         "          (5) : vivid(Not supported)\n" \
         "          (6) : colorswap(Not supported)\n" \
         "          (7) : solarize\n" \
         "          (8) : out-of-focus(Not supported)\n" \
         "          (9) : sky-blue(Not supported)\n" \
         "          (10) : grass-greeen(Not supported)\n" \
         "          (11) : skin-whiten(Not supported)\n"\
         "          (12) : noise(Not supported)\n" \
         "          (13) : emboss\n" \
         "          (14) : sketch(Not supported)\n" \
         "          (15) : oilpaint(Not supported)\n" \
         "          (16) : hatch(Not supported)\n" \
         "          (17) : gpen(Not supported)\n" \
         "          (18) : antialias(Not supported)\n" \
         "          (19) : dering(Not supported)\n" \
         "          (20) : posterize\n" \
         "          (21) : bw\n" \
         "          (22) : manual\n" \
         "          (23) : aqua\n"\
         "  Get colour-tone-mode Control:\n" \
         "      gft\n"
         "  Set Scene Mode:\n" \
         "      scm:<val> \n" \
         "          (0) : Manual\n" \
         "          (1) : Closeup(Not supported)\n" \
         "          (2) : Portrait\n" \
         "          (3) : Landscape\n" \
         "          (4) : Sports\n" \
         "          (5) : Night\n" \
         "          (6) : Auto\n"\
         "          (7) : Action\n" \
         "          (8) : Beach\n" \
         "          (9) : Candlelight\n" \
         "          (10) : Fireworks\n" \
         "          (11) : NightPortrait\n" \
         "          (12) : Party\n" \
         "          (13) : Snow\n" \
         "          (14) : Sunset\n" \
         "          (15) : Theatre\n" \
         "          (16) : Barcode\n"\
         "  Get Scene Mode:\n" \
         "      gscm\n"\
         "  Set Focus Mode:\n" \
         "      fm:<val> \n" \
         "          (0) : auto\n" \
         "          (1) : macro\n" \
         "          (2) : portrait(Not supported)\n" \
         "          (3) : infinity\n" \
         "          (4) : hyperfocal(Not supported)\n" \
         "          (5) : extended(Not supported)\n" \
         "          (6) : continuous-normal(Not supported)\n"\
         "          (7) : continuous-extended(Not supported)\n" \
         "          (8) : continuous-picture\n" \
         "          (9) : continuous-video\n" \
         "          (10) : fixed\n" \
         "  Get Focus Mode:\n" \
         "      gfm\n"\
         "  Set Noise-reduction Mode:\n" \
         "      nr:<val>, \n" \
         "          (1) : on\n" \
         "          (0) : off\n" \
         "  Get Noise-reduction Mode:\n" \
         "      gnr\n"
         "  Set Edge Enhancement:\n" \
         "      ee:<val> Range = -1.0 - 1.0, \n" \
         "  Get Edge Enhancement:\n" \
         "      gee\n"
         "  Set Audio Encoder:\n" \
         "      ae:<val>, val= 0(aac),1(amr)\n" \
         "  Get Audio Encoder:\n" \
         "      gae\n"
         "  Set Encoding Bit-rate(in bytes):\n"
         "      br:<val> eg br:4000000\n"
         "  Get Encoding Bit-rate(in bytes):\n"
         "      gbr\n"
         "  Set Encoding Rate control mode(0=CBR 1=VBR 2=VBR2):\n"
         "      rc:<val> eg rc:0\n"
         "  Get Encoding Rate control mode:\n"
         "      grc\n"
         " Set Temporal Tradeoff:\n"
         " tt:<val>, \n"
         "     (0) : 30fps\n"
         "     (1) : 24fps\n"
         "     (2) : 20fps\n"
         "     (3) : 15fps\n"
         "     (4) : 10fps\n"
         " Get Temporal Tradeoff:\n"
         " gtt\n"
         "  Set Frame Rate:\n"
         "      fr:<val> Range = 5 - 30, \n"
         "  Get Frame Rate:\n"
         "      gfr\n"
         "  Get Quantization Range:\n"
         "      gqp\n"
         //#endif
       */
      "  Set Container file type (0=mp4 1=3gp 2=avi):\n"
      "      k:<val> eg k:0\n"
      "  Get Container file type:\n"
      "      gk\n"
      "  Set Image Capture Resolution(only in image mode):\n"
      "      icr:<val> eg icr:3\n"
      "          (0) : 176 x 144\n"
      "          (1) : 320 x 240\n"
      "          (2) : 480 x 480\n"
      "          (3) : 640 x 480\n"
      "          (4) : 800 x 600\n"
      "          (5) : 720 x 480\n"
      "          (6) : 1280 x 720\n"
      "          (7) : 1280 x 960\n"
      "          (8) : 1600 x 1200\n"
      "          (9) : 2048 x 1536\n"
      "          (10) : 2240 x 1680\n"
      "          (11) : 2560 x 1920\n"
      "          (12) : 4096 x 3072 (only for t11x)\n"
      "  Get Image Capture Resolution:\n"
      "      gicr\n"
      "  Set Video Capture Resolution(only in video mode):\n"
      "      vcr:<val> eg vcr:3\n"
      "      * H.264/MPEG-4/VP8 (supports \"MP4/3GP/AVI\" container)\n"
      "          <0> : 176 x 144\n"
      "          <1> : 320 x 240\n"
      "          <3> : 640 x 480\n"
      "          <5> : 720 x 480\n"
      "          <6> : 1280 x 720\n"
      "          <7> : 1920 x 1080\n"
      "      * H.263 (supports \"MP4/3GP/AVI\" container)\n"
      "          <0> : 176 x 144 (QCIF)\n"
      "          <2> : 352 x 288 (CIF)\n"
      "          <4> : 704 x 576 (4CIF)\n"
      "      NOTE: USB camera capture only supports 176x144 resolution for H263 encode\n"
      "  Get Video Capture Resolution:\n"
      "      gvcr\n"
      "  Set Capture Device Node:\n"
      "      cdn:<val> Val = 0,1,2 (Capture device node)  \n"
      "  Get Capture Device Node:\n"
      "      gcdn\n"
      "  Set Video Encoder:\n"
      "      ve:<val>, val= 0(h264), 1(vp8), 2(mpeg4 software), 3(h263 software)\n"
      " Set Preview Sink Type: \n"
      "  --svs =nv_omx_videosink  for overlay, --svs =nv_omx_hdmi_videosink  for hdmi overlay, --svs =xvimagesink, --svs=ximagesink for ximagesink \n"
      "  Get Video Encoder:\n" "      gve\n\n");

  ctx = g_option_context_new ("Nvidia GStreamer Model Test");
  group = g_option_group_new ("Cotigao", NULL, NULL, NULL, NULL);
  g_option_group_add_entries (group, entries);
  g_option_context_set_description (ctx, nvgstcameraParam.extra_options);
  g_option_context_set_main_group (ctx, group);
  g_option_context_add_group (ctx, gst_init_get_option_group ());

  if (!g_option_context_parse (ctx, &argc, &argv, &error)) {
    g_option_context_free (ctx);
    g_error ("option parsing failed: %s", error->message);
  }

  if (!nvgstcameraParam.automate)
    g_print ("%s\n", nvgstcameraParam.extra_options);
  else
    g_print ("\nTo print help, enter -h\n\n");

  if (!nvgstcameraParam.file_name)
    nvgstcameraParam.file_name = g_strdup (NVGST_DEFAULT_FILENAME);
  g_option_context_free (ctx);

  gst_init (&argc, &argv);
  loop = g_main_loop_new (NULL, FALSE);

  nvgstcameraParam.lock = g_mutex_new ();
  nvgstcameraParam.cond = g_cond_new ();

#ifndef _INTERACTIVE
  channel = g_io_channel_unix_new (0);
  g_io_add_watch (channel, G_IO_IN, on_input, NULL);
#endif

  _intr_setup ();
  g_timeout_add (400, check_for_interrupt, NULL);

  if (nvgstVipParam.vip_enable) {
    if (create_vip_pipeline ()) {
      g_main_loop_run (loop);
      destroy_vip_pipeline ();
      g_mutex_free (nvgstcameraParam.lock);
      g_cond_free (nvgstcameraParam.cond);
      g_io_channel_unref (channel);
      g_main_loop_unref (loop);
    } else {
      destroy_vip_pipeline ();
      g_mutex_free (nvgstcameraParam.lock);
      g_cond_free (nvgstcameraParam.cond);
      g_io_channel_unref (channel);
      g_main_loop_unref (loop);
    }
  } else if (nvgstUsbCamParam.usbcam_enable || nvgstUsbCamParam.use_testsrc) {
    if (!nvgstUsbCamParam.vidcap_device)
      nvgstUsbCamParam.vidcap_device = g_strdup (NVGST_DEFAULT_VIDCAP_DEVICE);
    if (create_usbcam_pipeline ()) {
      g_main_loop_run (loop);
      destroy_usbcam_pipeline ();
      g_mutex_free (nvgstcameraParam.lock);
      g_cond_free (nvgstcameraParam.cond);
      g_io_channel_unref (channel);
      g_main_loop_unref (loop);
    } else {
      destroy_usbcam_pipeline ();
      g_mutex_free (nvgstcameraParam.lock);
      g_cond_free (nvgstcameraParam.cond);
      g_io_channel_unref (channel);
      g_main_loop_unref (loop);
    }
  } else if (nvgstCSIParam.csi_enable) {
    if (create_CSIcam_pipeline ()) {
      g_main_loop_run (loop);
      destroy_CSIcam_pipeline ();
      g_mutex_free (nvgstcameraParam.lock);
      g_cond_free (nvgstcameraParam.cond);
      g_io_channel_unref (channel);
      g_main_loop_unref (loop);
    } else {
      destroy_CSIcam_pipeline ();
      g_mutex_free (nvgstcameraParam.lock);
      g_cond_free (nvgstcameraParam.cond);
      g_io_channel_unref (channel);
      g_main_loop_unref (loop);
    }
  } else {

    if (nvgstcameraParam.automate == TRUE)
      g_timeout_add_seconds (nvgstcameraParam.capture_start_time, auto_capture,
          NULL);

#ifdef GST_KPI_MEASURE
    nvgstcameraParam.timer = g_timer_new ();
#endif
    if (create_pipeline ()) {
      g_main_loop_run (loop);
      destroy_pipeline ();
      g_mutex_free (nvgstcameraParam.lock);
      g_cond_free (nvgstcameraParam.cond);
      g_io_channel_unref (channel);
      g_main_loop_unref (loop);
    } else {
      destroy_pipeline ();
      g_mutex_free (nvgstcameraParam.lock);
      g_cond_free (nvgstcameraParam.cond);
      g_io_channel_unref (channel);
      g_main_loop_unref (loop);
    }
  }
  return ((nvgstcameraParam.return_value == -1) ? -1 : 0);
}
